#if !defined FISHER_H
#define FISHER_H

const int DEBUG = 2;

const int MAX_ITER = 100;

//---------------------------- global variable -------------------------------
const int BOOT_REP = 1;
const int BOOT_NO_REP = 2;
const int BOOT_ORI = 0;
//------- that can be set -----
const double ALLELE_FREQ_THRD = 0.0;

//----------------------- references of structures ------------------------------
//////////////////////////////////////////////////////////////////////////
// recombination rate in the table lkgen generated
struct LikTable
{
	int numHap;				// number of haplotypes that this table should be used to
	double *recombRate;		// different recombination rate in the table
	int numRate;			// number of different recombination rate
	int *comb;				// combinations in the table
	int numComb;			// number of combinations
	double *lik;			// likelihoods
};

//////////////////////////////////////////////////////////////////////////
// recombination rate that we should search
struct RateSearch
{
	double *rate;			// recombination rate that we search
	int nRate;				// number of recombination rate
	///// will not directly achieved
	double maxRate;		// maximal recombination rate searched
	double minRate;		// minimal geological recombination rate
};

//////////////////////////////////////////////////////////////////////////
// sequence
struct Sequ
{
	int *seq;		// sequence
	int numSamp;	// number of samples (diploid or haploid)
	int numSNP;		// number of SNPs
	int hg;			// diploid (2) or haploid (1)
};

//////////////////////////////////////////////////////////////////////////
// SNP locations
struct Loci
{
	int numSNP;		// number of SNPs
	double *locs;	// locations of SNPs (measured in kb)
};

//////////////////////////////////////////////////////////////////////////
// file names
struct FileNames
{
	char likFileName[100];	// likelihood table file name
	char inFoldName[100];	// input folder name
	char outFoldName[100];	// output folder name
	int beginFile;			// index of the first file proceeded
	int numFile;			// number of file that will be proceeded
	char seqFileName[100];	// sequence file name
	char locsFileName[100];	// locs file name
	char outFileName[100];	// output file name
};

//////////////////////////////////////////////////////////////////////////
// bootstrap parameters
struct BootPara
{
	int numBoot;	// times of bootstrap
	int bootType;	// bootstrap type
	int randSeed;	// random seed
};

//////////////////////////////////////////////////////////////////////////
// final likelihood matrix
struct DataLikMatrix
{
	int *typInd;		// genotype indice
	int *genoInd;		// genotype kinds
	double *likMatrix;	// likelihood matrix
	int numSNP;			// number of SNP
	int numRate;		// number of recombination rate in the likelihood table generated by lkgen
};

//////////////////////////////////////////////////////////////////////////
// weight vector
struct WinWeight
{
	int len;		// length of window
	double *wt;		// weight vector
	double grid;	// length of each grid gap
	int maxHotLen;		// = (max length of hotspot length) / grid
	int minHotLen;		// = (min length of hotspot length) / grid
	double thrd;		// threshould of likelihood to be judged as a hotspot
	int bkAll;			// use all SNPs to initialize background rate? Yes (1) or No (0)
	double *ewt;		// edge compensite weight
	double higher;		// times of higher to judge as hotspots
	double nsperc;		// if the previous LLR is lower than this percent of the thrd, then do not search
	double compLen;		// compensate about the length (so the longer hotspots are favoured)
};

//////////////////////////////////////////////////////////////////////////
// inter
struct Inter
{
	int st;
	int ed;
};

//////////////////////////////////////////////////////////////////////////
// grid rate
struct GridRate
{
	int *iRate;		// grid recombination rate index (per distance), length = numGrid - 1
	double *pos;	// grid position, length = numGrid
	int numGrid;		// number of grid point. Note: numGrid = numGap + 1; this setting is to correspond to numSNP
	double grid;		// distance between adjacent SNPs
};

//////////////////////////////////////////////////////////////////////////
// SNP rate
struct SNPRate
{
	double *aRate;		// accumulated recombination rate, length = numSNP - 1;
	double *locs;	// SNP position, length = numSNP
	int numSNP;			// number of SNPs
};

//////////////////////////////////////////////////////////////////////////
// information of a hotspot
struct HotInfo
{
	Inter hot;		// hotspot position
	int bkRate;		// background rate
	int hotRate;	// hotspot rate
	double lik;		// likelihood
	double stPos;	// start position
	double edPos;	// end position
	double dLik;	// delta likelihood
};

//////////////////////////////////////////////////////////////////////////
// information of hotspots in a region
struct HotReg
{
	int numGrid;	// number of grids
	int *hot;		// if hot[i] > 0, then it is a hotspot
	int *back;		// background
	int *free;		// region that can be searched as hotspot
	int *iRate;		// recombination rate in the region
	double *deltaLik;	// delta likelihood, with length = numGrid - 1
	int *edge;		// edge correction index
	int *iniBkTag;	// background rate initialized
	
	int numHot;		// number of hotspot founded
	Inter *hInter;	// hot inter 
	double *dLik;	// delta likelihood	(length = numGrid-1, efficient length = numHot+1)
	int *hotrate;	// hotspot rate;
	int *bkrate;	// background rate;
	Inter *sint;	// SNP interval
	int *edgecor;	// edge correction type
};

//////////////////////////////////////////////////////////////////////////
// information for potential new hotspot
struct NewHot
{
	int fl;				// if this hotspot is possible
	double dLik;		// delta likelihood
	Inter gint;		// hotspot position
	int bkRate;		// new background likelihood
	int hotRate;		// hotspot rate
	int edge;		// edge correction index

	Inter sint;		// hotspot interval in the sense of SNPs
	int cov;		// number of SNPs lies in the hotspot
//	double likAdd;	// the addition of likelihood, so that the longer hotspots are favored
};

//////////////////////////////////////////////////////////////////////////
// paramters for optimaze search
struct OptSr
{
	int dur;	// duration
	int xdur;	// interaction duration, smaller than dur
	int bkSt;	// background recombination rate searched at first
	int maxIter;	// maximize iteration times
	int ngap;
	double higher;
	double perc;
	int fn;
	int popu;
	double bkthrd;
	double amp;
	double hotperc;	// the maximum percentage that can be pre-set as hotspots
//	double likLenAdd;	// the likelihood added for each grid of length, so that long hotspots are favored
};


//////////////////////////////////////////////////////////////////////////
// an int vector
struct IntVect
{
	int num;
	int *vect;
};

#endif
