 #include <iostream.h>
#include <iomanip.h>
#include <fstream.h>
#include <string.h>
#include <stdlib.h>

#include "fisher.h"
#include "regliktool.h"
#include "mixliktool.h"
#include "truncliktool.h"
#include "vectool.h"
#include "readfile.h"
#include "linu.h"

//////////////////////////////////////////////////////////////////////////
// average recombination rate index that maximize the composite likelihood between two SNPs
int MixAverWholeRegRate(RateSearch& rs, Loci& lc, DataLikMatrix& lm, LikTable& lt, OptSr& os, WinWeight& ww)
{
	if (DEBUG != 2)
	{
		cout << "Initializing background rate..." << endl;
	}
	int i;
	int j;
	int k;
	int w;
	
	double lik;
	double maxLik;
	int maxInd;

	double *aRate = new double[lm.numSNP-1];	// aculmulated rate between each two continuous SNPs
	
	///// original rate
	for (i = 0; i < lm.numSNP-1; i++)
	{
		aRate[i] = rs.rate[os.bkSt] * (lc.locs[i+1] - lc.locs[i]);
	}
	maxInd = os.bkSt;
//	maxLik = TruncWholeRegLik(aRate, lm, lt, ww);
	maxLik = WholeRegLik(aRate, lm, lt);

	if (DEBUG != 2)
	{
		cout << "Beginning back ground rate = " << rs.rate[maxInd] << "\t\t\t" << "likelihood = " << maxLik << endl;
	}

	//////////////////////////////////////////////////////////////////////////
	// search rates
	int pn[2] = {-1, 1};
	int dur = 0;
	for (i = 1; i < rs.nRate; i++)
	{
		if (dur >= os.dur)
		{
			break;
		}
		
		int ifChange = 0;
		for (j = 0; j < 2; j++)
		{
			k = os.bkSt + pn[j] * i;
			if (k < 0 || k >= rs.nRate)
			{
				continue;
			}

			//////////////////////////////////////////////////////////////////////////
			// initialize the recombination rates
			for (w = 0; w < lm.numSNP-1; w++)
			{
				aRate[w] = rs.rate[k] * (lc.locs[w+1] - lc.locs[w]);
			}
			
			//////////////////////////////////////////////////////////////////////////
			// calculate the composite likelihood between SNP iS & jS
//			lik = TruncWholeRegLik(aRate, lm, lt, ww);
			lik = WholeRegLik(aRate, lm, lt);
			
			if (DEBUG != 2)
			{
				cout << "back ground rate = " << rs.rate[k] << "\t\t\t" << "likelihood = " << lik << endl;
			}

			if (lik > maxLik)
			{
				maxLik = lik;
				maxInd = k;
				ifChange = 1;
			}
		}
		
		if (ifChange == 1)
		{
			dur = 0;
		}
		else
		{
			dur++;
		}
	}

	if (DEBUG != 2)
	{
		cout << "Initialized back ground rate = " << rs.rate[maxInd] << "\t\t\t" << "likelihood = " << maxLik << endl;
	}

	delete[] aRate;
	return maxInd;
}

//////////////////////////////////////////////////////////////////////////
// initialize hotspot information of the whole region
void MixIniHotReg(HotReg& hr, GridRate& gr, SNPRate& sr, RateSearch& rs, Loci& lc, DataLikMatrix& lm, LikTable& lt, OptSr& os, WinWeight& ww)
{
	cout << "Initializing hot region..." << endl;

	int i;

	//!!!!!!if we want to set different background rate, we only need to change "for" below.
	//////////////////////////////////////////////////////////////////////////
	// set background tags
	MixSetBkTags(hr, gr, sr, rs, lc, lm, lt, os, ww);

	//////////////////////////////////////////////////////////////////////////
	// set free tags
	for (i = 0; i < hr.numGrid-1; i++)
	{
		hr.free[i] = 0;
	}

	//////////////////////////////////////////////////////////////////////////
	// set hot tags
	for (i = 0; i < hr.numGrid-1; i++)
	{
		hr.hot[i] = 0;
	}

	//////////////////////////////////////////////////////////////////////////
	// set rate tags
	int backRate = MixAverWholeRegRate(rs, lc, lm, lt, os, ww);
	for (i = 0; i < hr.numGrid-1; i++)
	{
		hr.iRate[i] = backRate;
	}
	backRate = MixMaxBkRate(hr, gr, sr, rs, lm, lt, os, ww);
	for (i = 0; i < hr.numGrid-1; i++)
	{
		hr.iRate[i] = backRate;
	}

	//////////////////////////////////////////////////////////////////////////
	// set delta likelihoods
	for (i = 0; i < hr.numGrid-1; i++)
	{
		hr.deltaLik[i] = 0.0;
	}

	cout << "OK!" << endl;

	//////////////////////////////////////////////////////////////////////////
	// set edge correction
	for (i = 0; i < hr.numGrid-1; i++)
	{
		hr.edge[i] = 0;
	}

	//////////////////////////////////////////////////////////////////////////
	// output
	if (DEBUG != 2)
	{
		OutHr(hr);
	}
}


//////////////////////////////////////////////////////////////////////////
// calculate background rate that maximize truncated pairwise likelihood
int MixMaxBkRate(HotReg& hr, GridRate& gr, SNPRate& sr, RateSearch& rs, DataLikMatrix& lm, LikTable& lt, OptSr& os, WinWeight& ww)
{
	int i, j, k, w;

	//////////////////////////////////////////////////////////////////////////
	// get background blocks
	Inter *sint = new Inter[gr.numGrid-1];
	int numBlock = GridBkBlock(sint, gr, hr, sr);

	int maxInd;
	double maxLik;
	double lik;
	
	//////////////////////////////////////////////////////////////////////////
	// original background rate
	int oriInd;
	for (i = 0; i < gr.numGrid-1; i++)
	{
		if (hr.back[i] == 0)
		{
			oriInd = hr.iRate[i];
			break;
		}
	}

	//////////////////////////////////////////////////////////////////////////
	// original likelihood
	for (i = 0; i < gr.numGrid-1; i++)
	{
		gr.iRate[i] = oriInd;
	}
	GridRate2SNPaRate(gr, sr, rs);

	maxLik = 0.0;
	for (i = 0; i < numBlock; i++)
	{
		if (sint[i].ed > sint[i].st)
		{
			maxLik += RegLik(sr.aRate, sint[i], lm, lt);
		}
	}
	maxInd = oriInd;

	//////////////////////////////////////////////////////////////////////////
	// search rates
	int dur = 0;
	int pn[2] = {-1, 1};
	for (i = 1; i < rs.nRate; i++)
	{
		if (dur >= os.dur)
		{
			break;
		}
		
		int ifChange = 0;
		for (j = 0; j < 2; j++)
		{
			k = oriInd + pn[j] * i;
			if (k < 0 || k >= rs.nRate)
			{
				continue;
			}
		
			//////////////////////////////////////////////////////////////////////////
			// set all the recombination rate to the trying bkrate
			for (w = 0; w < gr.numGrid-1; w++)
			{
				gr.iRate[w] = k;
			}
			GridRate2SNPaRate(gr, sr, rs);

			lik = 0.0;
			//////////////////////////////////////////////////////////////////////////
			// calculate the likelihood
			for (w = 0; w < numBlock; w++)
			{
				if (sint[w].ed > sint[w].st)
				{
//					lik += TruncRegLik(sr.aRate, sint[w], lm, lt, ww);
					lik += RegLik(sr.aRate, sint[w], lm, lt);
				}
			}

			if (DEBUG == 1)
			{
				cout << "back ground rate = " << k << "\t\t\t" << "likelihood = " << lik << endl;
			}

			if (lik > maxLik)
			{
				maxLik = lik;
				maxInd = k;
				ifChange = 1;
			}
		}

		if (ifChange == 1)
		{
			dur = 0;
		}
		else
		{
			dur++;
		}
	}

	delete[] sint;

	return maxInd;
}


//////////////////////////////////////////////////////////////////////////
// search the hotspot recombination rate that maximize the truncated pairwise likelihood of the whole region
int MixMaxHotRate(Inter& hot, HotReg& hr, GridRate& gr, SNPRate& sr, RateSearch& rs, DataLikMatrix& lm, LikTable& lt, OptSr& os, WinWeight& ww)
{
	int i;
	int j;
	int k;

	//////////////////////////////////////////////////////////////////////////
	// set original recombination rate
	for (i = 0; i < gr.numGrid-1; i++)
	{
		gr.iRate[i] = hr.iRate[i];
	}

	//////////////////////////////////////////////////////////////////////////
	// output
	if (DEBUG == 1)
	{
		cout << "Original recombination rate:" << endl;
		for (i = 0; i < gr.numGrid-1; i++)
		{
			cout << "\t" << gr.iRate[i];
		}
		cout << endl;
	}

	//////////////////////////////////////////////////////////////////////////
	// get the background recombination rate
	int oriInd;
	for (i = 0; i < gr.numGrid - 1; i++)
	{
		if (hr.back[i] == 0)
		{
			oriInd = hr.iRate[i];
			break;
		}
	}

	//////////////////////////////////////////////////////////////////////////
	// original likelihood
	int maxInd;
	double maxLik;
	double lik;
	for (i = hot.st; i < hot.ed; i++)
	{
		gr.iRate[i] = oriInd;
	}

	GridRate2SNPaRate(gr, sr, rs);
	maxLik = TruncWholeRegLik(sr.aRate, lm, lt, ww);
	maxInd = oriInd;

	int dur = 0;
	//////////////////////////////////////////////////////////////////////////
	// search new rates
	for (i = 1; i < rs.nRate; i++)
	{
		if (dur >= os.dur)
		{
			break;
		}
		
		k = oriInd + i;
		if (k >= rs.nRate)
		{
			break;
		}
		
		for (j = hot.st; j < hot.ed; j++)
		{
			gr.iRate[j] = k;
		}

		GridRate2SNPaRate(gr, sr, rs);
		lik = TruncWholeRegLik(sr.aRate, lm, lt, ww);

		if (DEBUG == 1)
		{
			cout << "back ground rate = " << k << "\t\t\t" << "likelihood = " << lik << endl;
		}

		if (lik > maxLik)
		{
			maxLik = lik;
			maxInd = k;
			dur = 0;
		}
		else
		{
			dur++;
		}
	}

	return maxInd;
}

//////////////////////////////////////////////////////////////////////////
// an experiental edge correction weight
void EdgeWt(WinWeight& ww)
{
	int i;
	int k;

	//////////////////////////////////////////////////////////////////////////
	// sum of weight
	double sum = 0.0;
	for (i = 0; i < ww.len; i++)
	{
		sum += ww.wt[i];
	}

	for (k = 0; k < ww.len-1; k++)
	{
		double sumL = 0.0;
		double par = 0;
		for (i = 0; i < ww.len; i++)
		{
			if (k >= i)
			{
				par = 1.0;
			}
			else
			{
				par = (k + 1.0) / (i + 1.0);
			}
			sumL += par * ww.wt[i];
		}
		ww.ewt[k] = sumL / sum;
	}
}

//////////////////////////////////////////////////////////////////////////
// iteration for new background rate
void MixInterBkRate(NewHot& nh, HotReg& hr, GridRate& gr, SNPRate& sr, RateSearch& rs, DataLikMatrix& lm, LikTable& lt, OptSr& os)
{
	int i;
	int j;
	int k;
	int w;

	//////////////////////////////////////////////////////////////////////////
	// set original recombination rate
	for (i = 0; i < gr.numGrid-1; i++)
	{
		gr.iRate[i] = hr.iRate[i];
	}

	//////////////////////////////////////////////////////////////////////////
	// original likelihood
	int maxInd;
	double maxLik;
	double lik;

	GridRate2SNPaRate(gr, sr, rs);
	maxLik = WholeRegLik(sr.aRate, lm, lt);
	maxInd = nh.bkRate;

	int dur = 0;
	int pn[2] = {-1, 1};
	//////////////////////////////////////////////////////////////////////////
	// search new rates
	for (i = 1; i < rs.nRate; i++)
	{
		if (dur >= os.xdur)
		{
			break;
		}
		
		int ifChange = 0;
		for (j = 0; j < 2; j++)
		{
			k = nh.bkRate + pn[j] * i;

			if (k >= nh.hotRate || k < 0)
			{
				continue;
			}
		
			for (w = 0; w < hr.numGrid - 1; w++)
			{
				if (hr.back[w] == 0)
				{
					gr.iRate[w] = k;
				}
			}

			GridRate2SNPaRate(gr, sr, rs);
			lik = WholeRegLik(sr.aRate, lm, lt);

			if (lik > maxLik)
			{
				maxLik = lik;
				maxInd = k;
				ifChange = 1;
			}
		}

		if (ifChange == 1)
		{
			dur = 0;
		}
		else
		{
			dur++;
		}
	}

	//////////////////////////////////////////////////////////////////////////
	// update
	if (maxInd != nh.bkRate)
	{
		nh.bkRate = maxInd;
		for (i = 0; i < hr.numGrid-1; i++)
		{
			if (hr.back[i] == 0)
			{
				hr.iRate[i] = nh.bkRate;
			}
		}
	}

	return;
}


//////////////////////////////////////////////////////////////////////////
// iteration for new background rate
void MixInterHotRate(NewHot& nh, HotReg& hr, GridRate& gr, SNPRate& sr, RateSearch& rs, DataLikMatrix& lm, LikTable& lt, OptSr& os)
{
	int i;
	int j;
	int k;
	int w;

	//////////////////////////////////////////////////////////////////////////
	// set original recombination rate
	for (i = 0; i < gr.numGrid-1; i++)
	{
		gr.iRate[i] = hr.iRate[i];
	}

	//////////////////////////////////////////////////////////////////////////
	// original likelihood
	int maxInd;
	double maxLik;
	double lik;

	GridRate2SNPaRate(gr, sr, rs);
	maxLik = WholeRegLik(sr.aRate, lm, lt);
	maxInd = nh.hotRate;

	int dur = 0;
	int pn[2] = {-1, 1};
	//////////////////////////////////////////////////////////////////////////
	// search new rates
	for (i = 1; i < rs.nRate; i++)
	{
		if (dur >= os.xdur)
		{
			break;
		}
		
		int ifChange = 0;
		for (j = 0; j < 2; j++)
		{
			k = nh.hotRate + pn[j] * i;

			if (k <= nh.bkRate || k < 0 || k >= rs.nRate)
			{
				continue;
			}
		
			for (w = nh.gint.st; w < nh.gint.ed; w++)
			{
				gr.iRate[w] = k;
			}

			GridRate2SNPaRate(gr, sr, rs);
			lik = WholeRegLik(sr.aRate, lm, lt);

			if (lik > maxLik)
			{
				maxLik = lik;
				maxInd = k;
				ifChange = 1;
			}
		}

		if (ifChange == 1)
		{
			dur = 0;
		}
		else
		{
			dur++;
		}
	}

	//////////////////////////////////////////////////////////////////////////
	// update
	if (maxInd != nh.hotRate)
	{
		nh.hotRate = maxInd;
		for (i = nh.gint.st; i < nh.gint.ed; i++)
		{
			hr.iRate[i] = nh.hotRate;
		}
	}

	return;
}


//////////////////////////////////////////////////////////////////////////
// calculate the mostfitted model while add one hotspot
void AddOneHot(NewHot& nh, HotReg& temphr, GridRate& gr, SNPRate& sr, RateSearch& rs, DataLikMatrix& lm, LikTable& lt, OptSr& os, WinWeight& ww)
{
	int i;

	/////////!!!!! whether to set its neighborhood as none background is under discussion
	for (i = nh.gint.st; i < nh.gint.ed; i++)
	{
		temphr.back[i] = 1;
	}

	//////////////////////////////////////////////////////////////////////////
	// background rate
	nh.bkRate = MixMaxBkRate(temphr, gr, sr, rs, lm, lt, os, ww);

	for (i = 0; i < temphr.numGrid-1; i++)
	{
		if (temphr.hot[i] == 0)
		{
			temphr.iRate[i] = nh.bkRate;
		}
	}

	//////////////////////////////////////////////////////////////////////////
	// original recombination rate
	for (i = 0; i < gr.numGrid-1; i++)
	{
		gr.iRate[i] = temphr.iRate[i];
	}
	GridRate2SNPaRate(gr, sr, rs);
	double oriLik = TruncWholeRegLik(sr.aRate, lm, lt, ww);		// original likelihood

	//////////////////////////////////////////////////////////////////////////
	// hotspot rate
	nh.hotRate = MixMaxHotRate(nh.gint, temphr, gr, sr, rs, lm, lt, os, ww);

	for (i = nh.gint.st; i < nh.gint.ed; i++)
	{
		temphr.iRate[i] = nh.hotRate;
	}
	
	if (DEBUG != 2)
	{
		cout << "\t\t" << nh.bkRate << " : " << nh.hotRate;
	}

	//////////////////////////////////////////////////////////////////////////
	// delta likelihood
	for (i = 0; i < gr.numGrid-1; i++)
	{
		gr.iRate[i] = temphr.iRate[i];
	}
	GridRate2SNPaRate(gr, sr, rs);
	double lik = TruncWholeRegLik(sr.aRate, lm, lt, ww);		// original likelihood
	nh.dLik = lik - oriLik;

	//////////////////////////////////////////////////////////////////////////
	// do edge compensation
	EdgeCor(nh, gr, sr, ww);

	return;
}

//////////////////////////////////////////////////////////////////////////
// do edge correction
void EdgeCor(NewHot& nh, GridRate& gr, SNPRate& sr, WinWeight& ww)
{
	int i;

	//////////////////////////////////////////////////////////////////////////
	// index from the right
	for (i = 1; i < sr.numSNP; i++)
	{
		if (gr.pos[nh.gint.st] < sr.locs[i])
		{
			break;
		}
	}
	if (i == sr.numSNP)
	{
		i -= 1;
	}
	i -= 1;
	int rInd = sr.numSNP-2 - i + 1;	// the index ("k", begin from 1) from the right

	//////////////////////////////////////////////////////////////////////////
	// index from the left
	for (i = sr.numSNP-1; i >= 0; i--)
	{
		if (gr.pos[nh.gint.ed] > sr.locs[i])
		{
			break;
		}
	}
	if (i == -1)
	{
		i += 1;
	}
	int lInd = i - 0 + 1;		// the index ("k", begin from 1) from the left

	//////////////////////////////////////////////////////////////////////////
	// final index
	int rdInd = lInd;
	if (rdInd > rInd)
	{
		rdInd = rInd;
	}
	if (rdInd < ww.len)
	{
		nh.edge = rdInd;
		nh.dLik /= ww.ewt[rdInd-1];
		if (DEBUG != 2)
		{
			cout << "\tEdge correction. edge index = " << rdInd << ".";
		}
	}
	else
	{
		nh.edge = 0;
	}

	return;
}

//////////////////////////////////////////////////////////////////////////
// part of initialize background tags
//void MixSetBkTags(HotReg& hr, GridRate& gr, SNPRate& sr, RateSearch& rs, Loci& lc, DataLikMatrix& lm, LikTable& lt, OptSr& os, WinWeight& ww)
//{
//	int i, j;
//
//	//////////////////////////////////////////////////////////////////////////
//	// sort background recombination rate
//	int *rate = new int[lc.numSNP - os.ngap];
//	for (i = 0; i < lc.numSNP - os.ngap; i++)
//	{
//		Inter sinter = {i, i + os.ngap};
//		rate[i] = AverRegRate(sinter, rs, lc, lm, lt);
//	}
//	int *ind = new int[lc.numSNP - os.ngap];
//	BubbleSortInd(ind, rate, lc.numSNP - os.ngap);
//	
//	//////////////////////////////////////////////////////////////////////////
//	// initialize background tags
//	for (i = 0; i < hr.numGrid-1; i++)
//	{
//		hr.back[i] = 0;
//	}
//
//	//////////////////////////////////////////////////////////////////////////
//	// set background tags
//	int sumGrid = 0;
//	int thrdGrid = int((gr.numGrid-1) * os.perc);
//	Inter ginter;
//	for (i = lc.numSNP - os.ngap - 1; i >= 0 ; i--)
//	{
//		//////////////////////////////////////////////////////////////////////////
//		// if enough, break
//		if (sumGrid >= thrdGrid)
//		{
//			break;
//		}
//
//		//////////////////////////////////////////////////////////////////////////
//		// starting and ending grid
//		ginter.st = int((sr.locs[ind[i]] + 0.0005 - gr.pos[0]) / gr.grid);
//		ginter.ed = int((sr.locs[ind[i] + os.ngap] - 0.0005 - gr.pos[0]) / gr.grid) + 1;
//		
//		//////////////////////////////////////////////////////////////////////////
//		// number of grid will be added
//		int add_num = 0;
//		for (j = ginter.st; j < ginter.ed; j++)
//		{
//			if (hr.back[j] == 0)
//			{
//				add_num++;
//			}
//		}
//		
//		//////////////////////////////////////////////////////////////////////////
//		// justify whether add this region
//		int if_add = 0;
//		if (sumGrid + add_num <= thrdGrid)
//		{
//			if_add = 1;
//		}
//		else if (sumGrid + add_num - thrdGrid <= thrdGrid - sumGrid)
//		{
//			if_add = 1;
//		}
//		else
//		{
//			if_add = 0;
//		}
//
//		//////////////////////////////////////////////////////////////////////////
//		// add it
//		if (if_add == 1)
//		{
//			for (j = ginter.st; j < ginter.ed; j++)
//			{
//				hr.back[j] = 1;
//			}
//			sumGrid += add_num;
//		}
//		else
//		{
//			break;
//		}
//	}
//
//	//////////////////////////////////////////////////////////////////////////
//	// set iniBkTags
//	for (i = 0; i < hr.numGrid-1; i++)
//	{
//		hr.iniBkTag[i] = hr.back[i];
//	}
//
//	delete[] rate;
//	delete[] ind;
//}



void MixSetBkTags(HotReg& hr, GridRate& gr, SNPRate& sr, RateSearch& rs, Loci& lc, DataLikMatrix& lm, LikTable& lt, OptSr& os, WinWeight& ww)
{
	int i, j;

	//////////////////////////////////////////////////////////////////////////
	// calculate the background recombination rate in sliding window
	int *rate = new int[lc.numSNP - os.ngap];
	for (i = 0; i < lc.numSNP - os.ngap; i++)
	{
		Inter sinter = {i, i + os.ngap};
		rate[i] = AverRegRate(sinter, rs, lc, lm, lt);
	}

	//////////////////////////////////////////////////////////////////////////
	// sort the rates
	int *ind = new int[lc.numSNP - os.ngap];
	BubbleSortInd(ind, rate, lc.numSNP - os.ngap);

	//////////////////////////////////////////////////////////////////////////
	// set the maximum rate of each window
	int *srate = new int[lc.numSNP-1];
	for (i = 0; i < lc.numSNP - os.ngap; i++)
	{
		for (j = 0; j < os.ngap; j++)
		{
			srate[ind[i] + j] = rate[i];
		}
	}
	int *sind = new int[lc.numSNP-1];
	BubbleSortInd(sind, srate, lc.numSNP-1);

	//============== this paragraph can be removed without change the function =============
	//////////////////////////////////////////////////////////////////////////
	// check how many percent is more than the threshold
	for (i = 0; i < lc.numSNP - 1; i++)
	{
		j = lc.numSNP - 2 - i;
		if (rs.rate[srate[j]] < os.bkthrd * os.amp)
		{
			break;
		}
	}
	double locsPerc = (i + 0.0) / (lc.numSNP - 1.0);
	double lenPerc = 0.0;
	for (j = 0; j < i; j++)
	{
		int k = lc.numSNP - 2 - j;
		lenPerc += (lc.locs[sind[k]+1] - lc.locs[sind[k]]);
	}
	lenPerc /= (lc.locs[lc.numSNP-1] - lc.locs[0]);
	cout << "Potential hot coverage: SNP percent = " << setiosflags(ios::fixed) << setprecision(3) << locsPerc << ";  "
		 << "length percent = " << setiosflags(ios::fixed) << setprecision(3) << lenPerc << "." << endl;
	//=============== end of this part ================================

	//////////////////////////////////////////////////////////////////////////
	// initialize background tags
	for (i = 0; i < hr.numGrid-1; i++)
	{
		hr.back[i] = 0;
	}

	//////////////////////////////////////////////////////////////////////////
	// set initial back tages
	Inter ginter;
	for (i = 0; i <= lc.numSNP * os.hotperc; i++)
	{
		j = lc.numSNP - 2 - i;
		if (rs.rate[srate[j]] >= os.bkthrd * os.amp)
		{
			//////////////////////////////////////////////////////////////////////////
			// starting and ending grid
			ginter.st = int((sr.locs[sind[j]] + 0.0005 - gr.pos[0]) / gr.grid);
			ginter.ed = int((sr.locs[sind[j] + 1] - 0.0005 - gr.pos[0]) / gr.grid) + 1;
			
			//////////////////////////////////////////////////////////////////////////
			// number of grid will be added
			for (j = ginter.st; j < ginter.ed; j++)
			{
				hr.back[j] = 1;
			}
		}
		else
		{
			break;
		}
	}

	//////////////////////////////////////////////////////////////////////////
	// set iniBkTags
	for (i = 0; i < hr.numGrid-1; i++)
	{
		hr.iniBkTag[i] = hr.back[i];
	}

	//////////////////////////////////////////////////////////////////////////
	// release space
	delete[] rate;
	delete[] ind;
	delete[] srate;
	delete[] sind;

	cout << "Initializing complete." << endl;
}



//void MixSetBkTags(HotReg& hr, GridRate& gr, SNPRate& sr, RateSearch& rs, Loci& lc, DataLikMatrix& lm, LikTable& lt, OptSr& os, WinWeight& ww)
//{
//	int i, j;
//
//	//////////////////////////////////////////////////////////////////////////
//	// sort background recombination rate
//	int *rate = new int[lc.numSNP - os.ngap];
//	for (i = 0; i < lc.numSNP - os.ngap; i++)
//	{
//		Inter sinter = {i, i + os.ngap};
//		rate[i] = AverRegRate(sinter, rs, lc, lm, lt);
//	}
//	int *ind = new int[lc.numSNP - os.ngap];
//	BubbleSortInd(ind, rate, lc.numSNP - os.ngap);
//	
//	//////////////////////////////////////////////////////////////////////////
//	// initialize background tags
//	for (i = 0; i < hr.numGrid-1; i++)
//	{
//		hr.back[i] = 0;
//	}
//
//	//////////////////////////////////////////////////////////////////////////
//	// set background tags
//	Inter ginter;
//	double *dRate = new double[gr.numGrid-1];
//	for (i = 0; i < gr.numGrid-1; i++)
//	{
//		dRate[i] = 0.0;
//	}
//	for (i = lc.numSNP - os.ngap - 1; i >= 0 ; i--)
//	{
//		//////////////////////////////////////////////////////////////////////////
//		// starting and ending grid
//		ginter.st = int((sr.locs[ind[i]] + 0.0005 - gr.pos[0]) / gr.grid);
//		ginter.ed = int((sr.locs[ind[i] + os.ngap] - 0.0005 - gr.pos[0]) / gr.grid) + 1;
//		
//		//////////////////////////////////////////////////////////////////////////
//		// number of grid will be added
//		for (j = ginter.st; j < ginter.ed; j++)
//		{
//			if (hr.back[j] == 0)
//			{
//				dRate[j] = rs.rate[rate[i]];
//			}
//			hr.back[j] = 1;
//		}
//	}
//
//	//////////////////////////////////////////////////////////////////////////
//	// set iniBkTags
//	for (i = 0; i < hr.numGrid-1; i++)
//	{
//		if (dRate[i] >= os.bkthrd * os.amp)
//		{
//			hr.back[i] = 1;
//		}
//		else
//		{
//			hr.back[i] = 0;
//		}
//	}
//
//	for (i = 0; i < hr.numGrid-1; i++)
//	{
//		hr.iniBkTag[i] = hr.back[i];
//	}
//
//	//////////////////////////////////////////////////////////////////////////
//	// average background rate
//	int bkRate = MixAverWholeRegRate(rs, lc, lm, lt, os, ww);
//
//	delete[] rate;
//	delete[] ind;
//	delete[] dRate;
//
//	cout << "Initializing complete." << endl;
//}
//
//////////////////////////////////////////////////////////////////////////
// part of initialize background tags
//void MixSetBkTags(HotReg& hr, GridRate& gr, SNPRate& sr, RateSearch& rs, Loci& lc, DataLikMatrix& lm, LikTable& lt, OptSr& os, WinWeight& ww)
//{
//	int i, j;
//
//	int bkRate = MixAverWholeRegRate(rs, lc, lm, lt, os, ww);
//	
//	//////////////////////////////////////////////////////////////////////////
//	// background measured in SNPs
//	int *back = new int[lc.numSNP-1];
//	for (i = 0; i < lc.numSNP - 1; i++)
//	{
//		back[i] = 0;
//	}
//	for (i = 0; i < lc.numSNP - os.ngap; i++)
//	{
//		Inter sinter = {i, i + os.ngap};
//		int iRate = AverRegRate(sinter, rs, lc, lm, lt);
//		if (rs.rate[iRate] > os.higher * rs.rate[bkRate])
//		{
//			for (j = i; j < i + os.ngap; j++)
//			{
//				back[j] = 1;
//			}
//		}
//	}
//	
//	//////////////////////////////////////////////////////////////////////////
//	// background measured in grids
//	for (i = 0; i < hr.numGrid-1; i++)
//	{
//		hr.back[i] = 0;
//	}
//	Inter ginter;
//	for (i = 0; i < lc.numSNP-1; i++)
//	{
//		if (back[i] == 1)
//		{
//			ginter.st = int((sr.locs[i] + 0.0005 - gr.pos[0]) / gr.grid);
//			ginter.ed = int((sr.locs[i+1] - 0.0005 - gr.pos[0]) / gr.grid) + 1;
//			for (j = ginter.st; j < ginter.ed; j++)
//			{
//				hr.back[j] = 1;
//			}
//		}
//	}
//
//	//////////////////////////////////////////////////////////////////////////
//	// set iniBkTags
//	for (i = 0; i < hr.numGrid-1; i++)
//	{
//		hr.iniBkTag[i] = hr.back[i];
//	}
//
//	delete[] back;
//}
//


//////////////////////////////////////////////////////////////////////////
// judge if available
int HotAvailable(Inter gint, HotReg& hr, GridRate& gr, SNPRate& sr)
{
	int i;

	if (gr.pos[gint.ed] > sr.locs[sr.numSNP-1] + 0.0005)
	{
		return 0;
	}

	if (gint.ed > gr.numGrid-1)
	{
		return 0;
	}

	int ifFree = 0;
	for (i = gint.st; i < gint.ed; i++)
	{
		if (hr.free[i] != 0)
		{
			ifFree = 1;
			break;
		}
	}
	if (ifFree == 1)
	{
		return 0;
	}

	return 1;
}

//////////////////////////////////////////////////////////////////////////
// output delta likelihood matrix
void OutdLikMatrix(HotReg& hr, NewHot* nHot, WinWeight& ww)
{
	cout << "delta likelihood matrix is:" << endl;
	int pos;
	int len;
	for (pos = 0; pos < hr.numGrid-1; pos++)
	{
		for (len = ww.minHotLen; len <= ww.maxHotLen; len++)
		{
			int iHot = Ind2(pos, len-ww.minHotLen, (ww.maxHotLen-ww.minHotLen+1));
			if (nHot[iHot].fl == 0)
			{
				cout << "\t" << setiosflags(ios::fixed) << setw(8) << setprecision(1) << -1.0;
			}
			else
			{
				cout << "\t" << setiosflags(ios::fixed) << setw(8) << setprecision(1) << nHot[iHot].dLik;
			}
		}
		cout << endl;
	}
}

//////////////////////////////////////////////////////////////////////////
// find index of maximal delta-likelihood
int MaxdLikInd(HotReg& hr, NewHot* nHot, WinWeight& ww)
{
	int maxI = -1;
	double maxdLik = 0.0;
	int numPossiHot = (hr.numGrid-1) * (ww.maxHotLen-ww.minHotLen+1);
//	for (int iHot = 0; iHot < numPossiHot; iHot++)
//	{
//		if (nHot[iHot].fl == 1 && nHot[iHot].dLik + nHot[iHot].likAdd > maxdLik)
//		{
//			maxI = iHot;
//			maxdLik = nHot[iHot].dLik + nHot[iHot].likAdd;
//		}
//	}

	for (int iHot = 0; iHot < numPossiHot; iHot++)
	{
		int ifAcc = 0;
		if (nHot[iHot].fl == 1)
		{
			if (nHot[iHot].gint.ed - nHot[iHot].gint.st > nHot[maxI].gint.ed - nHot[maxI].gint.st)
			{
				if (nHot[iHot].dLik + ww.compLen > maxdLik)
				{
					ifAcc = 1;
				}
			}
			else if (nHot[iHot].gint.ed - nHot[iHot].gint.st == nHot[maxI].gint.ed - nHot[maxI].gint.st)
			{
				if (nHot[iHot].dLik > maxdLik)
				{
					ifAcc = 1;
				}
			}
			else
			{
				if (nHot[iHot].dLik - ww.compLen > maxdLik)
				{
					ifAcc = 1;
				}
			}
		}
		if (ifAcc == 1)
		{
			maxI = iHot;
			maxdLik = nHot[iHot].dLik;
		}
	}

	return maxI;
}

//////////////////////////////////////////////////////////////////////////
// update hr
void UpdateHr(HotReg& hr, NewHot& nh, int iter)
{
	int i;

	//////////////////////////////////////////////////////////////////////////
	// hotspot information
	hr.hInter[hr.numHot].st = nh.gint.st;
	hr.hInter[hr.numHot].ed = nh.gint.ed;
	
	//////////////////////////////////////////////////////////////////////////
	// delta likelihood
	for (i = nh.gint.st; i < nh.gint.ed; i++)
	{
		hr.deltaLik[i] = nh.dLik;
	}
	hr.dLik[hr.numHot] = nh.dLik;
	hr.bkrate[hr.numHot] = nh.bkRate;
	hr.hotrate[hr.numHot] = nh.hotRate;
	hr.sint[hr.numHot] = nh.sint;
	hr.edgecor[hr.numHot] = nh.edge;
	hr.numHot++;

	//////////////////////////////////////////////////////////////////////////
	// recombination rates
	for (i = 0; i < hr.numGrid-1; i++)
	{
		if (hr.hot[i] == 0)
		{
			hr.iRate[i] = nh.bkRate;
		}
	}
	for (i = nh.gint.st; i < nh.gint.ed; i++)
	{
		hr.iRate[i] = nh.hotRate;
	}

	//////////////////////////////////////////////////////////////////////////
	// background tags
	for (i = nh.gint.st; i < nh.gint.ed; i++)
	{
		hr.back[i] = 1;
	}

	//////////////////////////////////////////////////////////////////////////
	// hotspot tags
	for (i = nh.gint.st; i < nh.gint.ed; i++)
	{
		hr.hot[i] = iter + 1;
	}

	//////////////////////////////////////////////////////////////////////////
	// free tags
	for (i = nh.gint.st; i < nh.gint.ed; i++)
	{
		hr.free[i] = 1;
	}
	if (nh.gint.st != 0)
	{
		hr.free[nh.gint.st - 1] = 1;
	}
	if (nh.gint.ed != hr.numGrid-1)
	{
		hr.free[nh.gint.ed] = 1;
	}

	//////////////////////////////////////////////////////////////////////////
	// edge correction tags
	for (i = nh.gint.st; i < nh.gint.ed; i++)
	{
		hr.edge[i] = nh.edge;
	}
}

//////////////////////////////////////////////////////////////////////////
// output hr
void OutHr(HotReg& hr)
{
	int i;

	//////////////////////////////////////////////////////////////////////////
	// background tags
	cout << "Background tags are:" << endl;
	for (i = 0; i < hr.numGrid-1; i++)
	{
		cout << "\t" << hr.back[i];
	}
	cout << endl;

	//////////////////////////////////////////////////////////////////////////
	// free to search tags
	cout << "Free to search tags are:" << endl;
	for (i = 0; i < hr.numGrid-1; i++)
	{
		cout << "\t" << hr.free[i];
	}
	cout << endl;

	//////////////////////////////////////////////////////////////////////////
	// hot tags
	cout << "Hot tags are:" << endl;
	for (i = 0; i < hr.numGrid-1; i++)
	{
		cout << "\t" << hr.hot[i];
	}
	cout << endl;

	//////////////////////////////////////////////////////////////////////////
	// delta likelihoods
	cout << "Delta likelihoods are:" << endl;
	for (i = 0; i < hr.numGrid-1; i++)
	{
		cout << "\t" << hr.deltaLik[i];
	}
	cout << endl;

	//////////////////////////////////////////////////////////////////////////
	// recombination rates
	cout << "Recombination rates are:" << endl;
	for (i = 0; i < hr.numGrid-1; i++)
	{
		cout << "\t" << hr.iRate[i];
	}
	cout << endl;
}

//////////////////////////////////////////////////////////////////////////
// get the background blocks
int GridBkBlock(Inter* sint, GridRate& gr, HotReg& hr, SNPRate& sr)
{
	int i, j, k;
	//////////////////////////////////////////////////////////////////////////
	// output background tags
	if (DEBUG == 1)
	{
		cout << "The background tags are:" << endl;
		for (i = 0; i < gr.numGrid-1; i++)
		{
			cout << "\t" << hr.back[i];
		}
		cout << endl;
	}

	//////////////////////////////////////////////////////////////////////////
	// get the background grid blocks
	int numBlock = 0;
	Inter *gint = new Inter[gr.numGrid-1];	// backgound inter
	for (i = 0; i < gr.numGrid-1; i++)
	{
		//// the first grid
		if (i == 0 && hr.back[i] == 0)
		{
			gint[numBlock].st = 0;
		}
		//// others
		if (i != 0)
		{
			if (hr.back[i] == 0 && hr.back[i-1] == 1)
			{
				gint[numBlock].st = i;
			}
			else if (hr.back[i] == 1 && hr.back[i-1] == 0)
			{
				gint[numBlock].ed = i;
				numBlock++;
			}
		}
		//// the last grid 
		if (i == gr.numGrid - 2 && hr.back[i] == 0)
		{
			gint[numBlock].ed = gr.numGrid - 1;
			numBlock++;
		}
	}
	///// output it
	if (DEBUG == 1)
	{
		cout << numBlock << " background blocks." << endl
			 << "Their grid startings and endings are:" << endl;
		for (i = 0; i < numBlock; i++)
		{
			cout << gint[i].st << "\t" << gint[i].ed << endl;
		}
		cout << endl;
	}

	//////////////////////////////////////////////////////////////////////////
	// get the background SNP blocks
	for (j = 0; j < numBlock; j++)
	{
		//// start
		for (i = 0; i < sr.numSNP; i++)
		{
			if (sr.locs[i] > gr.pos[gint[j].st] - 0.0005)
			{
				break;
			}
		}
		sint[j].st = i;
		//// end
		for (k = i; k < sr.numSNP; k++)
		{
			if (sr.locs[k] > gr.pos[gint[j].ed] + 0.0005)
			{
				break;
			}
		}
		sint[j].ed = k - 1;
		//// edge
		if (k == sr.numSNP-1 && sr.locs[k] > gr.pos[gint[j].ed] - 0.0005)
		{
			sint[j].ed = k;
		}
	}
	
	//// output
	if (DEBUG == 1)
	{
		cout << numBlock << " background blocks." << endl
			 << "Their SNP startings and endings are:" << endl;
		for (i = 0; i < numBlock; i++)
		{
			cout << sint[i].st << "\t" << sint[i].ed << endl;
		}
	}

	delete[] gint;
	return numBlock;
}

//////////////////////////////////////////////////////////////////////////
//	initalize hotspot positions
//////////////////////////////////////////////////////////////////////////
void IniPossiHot(NewHot *nHot, Loci& lc, WinWeight& ww, GridRate& gr, OptSr& os)
{
	int pos;
	int len;

	for (pos = 0; pos < gr.numGrid-1; pos++)
	{
		for (len = ww.maxHotLen; len >= ww.minHotLen; len--)
		{
			//////////////////////////////////////////////////////////////////////////
			// position in the sense of grid
			int i = Ind2(pos, ww.maxHotLen-len, (ww.maxHotLen-ww.minHotLen+1));
			nHot[i].gint.st = pos;
			nHot[i].gint.ed = pos + len;

			//////////////////////////////////////////////////////////////////////////
			// position in the sense of SNPs
			nHot[i].sint.st = LocGridSNP(nHot[i].gint.st, lc, gr);
			nHot[i].sint.ed = LocGridSNP(nHot[i].gint.ed, lc, gr);
			nHot[i].cov = nHot[i].sint.ed - nHot[i].sint.st;

//			nHot[i].likAdd = os.likLenAdd * len;
		}
	}

	return;
}

//////////////////////////////////////////////////////////////////////////
// the grid position in the sense of SNPs
//////////////////////////////////////////////////////////////////////////
int LocGridSNP(int gNum, Loci& lc, GridRate& gr)
{
	int i;
	for (i = 0; i < lc.numSNP; i++)
	{
		if (lc.locs[i] > gr.pos[gNum] + 0.0005)
		{
			break;
		}
	}
	
	//////////////////////////////////////////////////////////////////////////
	// to avoid out of indice
	if (i == lc.numSNP)
	{
		i = lc.numSNP - 1;
	}

	return (i - 1);
}

//////////////////////////////////////////////////////////////////////////
// copy hotspot
//////////////////////////////////////////////////////////////////////////
void CopyNewHot(NewHot& nnh, NewHot& nh, RateSearch& rs)
{
	nnh.bkRate = nh.bkRate;
	nnh.dLik = nh.dLik;
	nnh.edge = nh.edge;

	double hotrate = rs.rate[nh.hotRate] * (nh.gint.ed - nh.gint.st);
	hotrate += (rs.rate[nh.bkRate] * ((nnh.gint.ed - nnh.gint.st) - (nh.gint.ed - nh.gint.st)));
	hotrate /= (nnh.gint.ed - nnh.gint.st);

	for (int i = 0; i < rs.nRate; i++)
	{
		if (hotrate <= rs.rate[i])
		{
			nnh.hotRate = i;
			break;
		}
	}
	if (hotrate > rs.rate[rs.nRate - 1])
	{
		nnh.hotRate = rs.nRate - 1;
	}
	return;
}


//////////////////////////////////////////////////////////////////////////
//
//		main routine to search recombination hotspots
//
//////////////////////////////////////////////////////////////////////////
void MixHot(RateSearch& rs, Loci& lc, DataLikMatrix& lm, LikTable& lt, WinWeight& ww, HotReg& hr, OptSr& os, FileNames& fn)
{
	cout << endl << "Searching recombination hotspots..." << endl;

	int i;

	//////////////////////////////////////////////////////////////////////////
	// initialize Grid rate
	GridRate gr;
	CalNumGrid(gr, lc, ww);
	gr.pos = new double[gr.numGrid];
	gr.iRate = new int[gr.numGrid - 1];
	IniGrid(gr, lc, ww);

	//////////////////////////////////////////////////////////////////////////
	// initialize SNP rate
	SNPRate sr;
	sr.numSNP = lc.numSNP;
	sr.locs = new double[sr.numSNP];
	sr.aRate = new double[sr.numSNP - 1];
	IniSNPRate(sr, lc);

	//////////////////////////////////////////////////////////////////////////
	// initalize hotspot information
	hr.numGrid = gr.numGrid;
	hr.back = new int[hr.numGrid-1];
	hr.free = new int[hr.numGrid-1];
	hr.hot = new int[hr.numGrid-1];
	hr.iRate = new int[hr.numGrid-1];
	hr.deltaLik = new double[hr.numGrid-1];
	hr.edge = new int[hr.numGrid-1];
	hr.iniBkTag = new int[hr.numGrid-1];

	hr.numHot = 0;
	hr.hInter = new Inter[hr.numGrid-1];
	hr.dLik = new double[hr.numGrid-1];
	hr.bkrate = new int[hr.numGrid-1];
	hr.hotrate = new int[hr.numGrid-1];
	hr.sint = new Inter[hr.numGrid-1];
	hr.edgecor = new int[hr.numGrid-1];

	MixIniHotReg(hr, gr, sr, rs, lc, lm, lt, os, ww);

	//////////////////////////////////////////////////////////////////////////
	// searching recombination rate
	int numPossiHot = (gr.numGrid-1) * (ww.maxHotLen-ww.minHotLen+1);
	NewHot *nHot = new NewHot[numPossiHot];	// new hotspot
	IniPossiHot(nHot, lc, ww, gr, os);

	HotReg temphr;
	temphr.numGrid = hr.numGrid;
	temphr.back = new int[temphr.numGrid-1];
	temphr.free = new int[temphr.numGrid-1];
	temphr.hot = new int[temphr.numGrid-1];
	temphr.iRate = new int[temphr.numGrid-1];

	int iter;
	for (iter = 0; iter < MAX_ITER; iter++)
	{
		if (DEBUG != 2)
		{
			cerr << endl << endl << "ITERATION " << iter << "." << endl;
		}
		else
		{
			cout << endl << endl << "ITERATION " << iter << "." << endl;
		}
		
		//////////////////////////////////////////////////////////////////////////
		// searching all possible hotspots
		for (int iHot = 0; iHot < numPossiHot; iHot++)
		{
			if (DEBUG != 2)
			{
				cout << "Start grid = " << nHot[iHot].gint.st << "; End grid = " << nHot[iHot].gint.ed << ".";
			}
			//////////////////////////////////////////////////////////////////////////
			// judge if the location of hotspot is permitted
			nHot[iHot].fl = HotAvailable(nHot[iHot].gint, hr, gr, sr);

			if (nHot[iHot].fl == 0)
			{
				if (DEBUG != 2)
				{
					cout << "\tNot available." << endl;
				}
				continue;
			}

			//////////////////////////////////////////////////////////////////////////
			// judge whether the previous LLR is too low
			if (iter > 0 && nHot[iHot].dLik <= ww.thrd * ww.nsperc)
			{
				if (DEBUG != 2)
				{
					cout << "\tToo low LLR, need not search." << endl;
				}
				continue;
			}

			//////////////////////////////////////////////////////////////////////////
			// check whether this hotspot is not needed to be calculated
			if (nHot[iHot].cov == 0)
			{
				for (i = 0; i < iHot; i++)
				{
					if (nHot[i].fl == 1 && nHot[i].cov == 0 && nHot[i].sint.st == nHot[iHot].sint.st)
					{
						CopyNewHot(nHot[iHot], nHot[i], rs);
						break;
					}
				}
			}
			else
			{
				//////////////////////////////////////////////////////////////////////////
				// calculate the most fitted model
				for (i = 0; i < hr.numGrid-1; i++)
				{
					temphr.back[i] = hr.back[i];
					temphr.free[i] = hr.free[i];
					temphr.hot[i] = hr.hot[i];
					temphr.iRate[i] = hr.iRate[i];
				}

				//----------------- key function -------------------//
				AddOneHot(nHot[iHot], temphr, gr, sr, rs, lm, lt, os, ww);
			}
				
			if (DEBUG != 2)
			{
				cout << endl;
			}
		}

		//////////////////////////////////////////////////////////////////////////
		// output delta likelihood matrix
		if (DEBUG != 2)
		{
			OutdLikMatrix(hr, nHot, ww);
		}

		//////////////////////////////////////////////////////////////////////////
		// find the maximal delta likelihood
		int maxI = MaxdLikInd(hr, nHot, ww);
		if (maxI == -1)
		{
			cout << endl << "End of iteration." << endl;
			break;
		}

		if (DEBUG != 2)
		{
			cout << "Potential hotspot that maximize delta likelihood is:\t"
				 << "start = " << nHot[maxI].gint.st << "\tend = "<< nHot[maxI].gint.ed << "\tdelta likelihood = " << nHot[maxI].dLik << endl;
		}

		//////////////////////////////////////////////////////////////////////////
		// update if necessary
		if (nHot[maxI].dLik >= ww.thrd)
		{
			cout << "Start = " << gr.pos[nHot[maxI].gint.st] << ",\tEnd = " << gr.pos[nHot[maxI].gint.ed] << ",\t"
				 << "Delta likelihood = " << nHot[maxI].dLik << endl
				 << "The new hotspot has been accepted." << endl;
			UpdateHr(hr, nHot[maxI], iter);

			//////////////////////////////////////////////////////////////////////////
			// output results
			if (DEBUG != 2)
			{
				OutHr(hr);
			}
		}
		else
		{
			cout << "Start = " << gr.pos[nHot[maxI].gint.st] << ",\tEnd = " << gr.pos[nHot[maxI].gint.ed] << ",\t"
				 << "Delta likelihood = " << nHot[maxI].dLik << endl
				 << "The new hotspot has been rejected." << endl;
			hr.dLik[hr.numHot] = nHot[maxI].dLik;
			cout << endl << "End of iteration." << endl;
			break;
		}
	}

	//////////////////////////////////////////////////////////////////////////
	// write results to file
	WriteRes(fn, lc, rs, hr, ww);

	//////////////////////////////////////////////////////////////////////////
	// release space
	delete[] hr.back;
	delete[] hr.free;
	delete[] hr.hot;
	delete[] hr.iRate;
	delete[] hr.hInter;
	delete[] hr.dLik;
	delete[] hr.deltaLik;
	delete[] hr.edge;
	delete[] hr.iniBkTag;
	delete[] hr.hotrate;
	delete[] hr.bkrate;
	delete[] hr.sint;
	delete[] hr.edgecor;

	//////////////////////////////////////////////////////////////////////////
	// release space
	delete[] gr.pos;
	delete[] gr.iRate;
	delete[] sr.locs;
	delete[] sr.aRate;
	
	delete[] temphr.back;
	delete[] temphr.free;
	delete[] temphr.hot;
	delete[] temphr.iRate;

	delete[] nHot;
}
