#include <iostream.h>
#include <fstream.h>
#include <iomanip.h>
#include <stdlib.h>
#include "fisher.h"
#include "readfile.h"
#include "vectool.h"

//////////////////////////////////////////////////////////////////////////
//	read in the basic parameters of the likelihood file
//  set:
//	numHap, numComb, numRate
void ReadBasicPara(ifstream& likFile, LikTable& lt)
{
	int iTemp;
	double dTemp;

	cout << "Reading basic parameters from likelihood file... ";
	likFile >> lt.numHap >> lt.numComb >> iTemp >> dTemp >> lt.numRate >> dTemp;
	cout << "OK!" << endl;

	if (DEBUG == 1)
	{
		cout << "numHap = " << lt.numHap << endl
			 << "numComb = " << lt.numComb << endl
			 << "numRate = " << lt.numRate << endl;
	}
}

//////////////////////////////////////////////////////////////////////////
// read in the recombination rate
// set:
// recombRate
void ReadRecombRate(ifstream& likFile, LikTable& lt)
{
	char cTemp;
	int i;

	cout << "Reading the recombination rates... ";
	while (1)
	{
		likFile >> cTemp;
		if (cTemp == 'o')	// find the end of 'Rho'
		{
			break;
		}
	}
	for (i = 0; i < lt.numRate; i++)
	{
		likFile >> lt.recombRate[i];
	}
	cout << "OK!" << endl;

	if (DEBUG == 1)
	{
		for (i = 0; i < lt.numRate; i++)
		{
			cout << lt.recombRate[i] << "\t";
		}
		cout << endl;
	}
}

//////////////////////////////////////////////////////////////////////////
// retouch the recombination rate
// reset:
// recombRate
void RetouchRecombRate(LikTable& lt)
{
	int i;
	for (i = 0; i < lt.numRate-1; i++)
	{
		lt.recombRate[i] = lt.recombRate[lt.numRate-1] * i / (lt.numRate-1.0);
	}
		
	if (DEBUG == 1)
	{
		cout << "The retouched recombination rates are:\t";
		for (i = 0; i < lt.numRate; i++)
		{
			cout << lt.recombRate[i] << "\t";
		}
		cout << endl;
	}
}

//////////////////////////////////////////////////////////////////////////
// read in the combinations and the likelihoods
// set:
// comb, lik
void ReadLik(ifstream& likFile, LikTable& lt)
{
	cout << "Reading the haplotype combinations and the likelihoods... ";
	char cTemp;
	int i, j;

	for (i = 0; i < lt.numComb; i++)
	{
		while (1)
		{
			cTemp = likFile.get();
			if (cTemp == '#')
			{
				break;
			}
		}
		for (j = 0; j < 4; j++)
		{
			likFile >> lt.comb[Ind2(i, j, 4)];
		}
		while (1)
		{
			cTemp = likFile.get();
			if (cTemp == ':')
			{
				break;
			}
		}
		for (j = 0; j < lt.numRate; j++)
		{
			likFile >> lt.lik[Ind2(i, j, lt.numRate)];
		}
	}
	cout << "OK!" << endl;

	if (DEBUG == 1)
	{
		ofstream likFileOut("likelihood.txt");
//		if (likFileOut.fail())
//		{
//			cout << "Error in open likelihood.txt to write likelihood!" << endl;
//			exit(0);
//		}
		for (i = 0; i < lt.numComb; i++)
		{
			for (j = 0; j < 4; j++)
			{
				likFileOut << setiosflags(ios::fixed) << setw(4) << setprecision(1) << lt.comb[Ind2(i, j, 4)] << "\t";
			}
			likFileOut << ":";
			for (j = 0; j < lt.numRate; j++)
			{
				likFileOut << "\t" << setiosflags(ios::fixed) << setw(9) << setprecision(2) << lt.lik[Ind2(i, j, lt.numRate)];
			}
			likFileOut << endl;
		}
	}
}

//////////////////////////////////////////////////////////////////////////
// read in parameters from sequence file
// set:
// numSamp, numSNP, hg
void ReadSeqPara(ifstream& seqFile, Sequ& sq)
{
	cout << "Reading sequence file parameters... ";
	seqFile >> sq.numSamp >> sq.numSNP >> sq.hg;

	cout << "OK!  ";
	if (sq.hg == 2)
	{
		cout << sq.numSamp << " diploid samples.  ";
	}
	else
	{
		cout << sq.numSamp << " haploid samples.  ";
	}
	cout << sq.numSNP << " SNPs." << endl;
}

//////////////////////////////////////////////////////////////////////////
// read in genotype sequence from sequence file
// set:
// seq
void ReadSeq(ifstream& seqFile, Sequ& sq)
{
	cout << "Reading sequence... " << endl;
	int i = -1;	// number of sequence
	int j = 0;	// number of SNP
	char cTemp;
	while (!seqFile.eof())
	{
		cTemp = seqFile.get();
		if (cTemp == '>')
		{
			i++;
			j = 0;	// set to the begining of another sequence
			if (DEBUG != 2)
			{
				cout << "Reading sequence " << i << "." << endl;
			}
			// masking the sequence name line
			while (!seqFile.eof())
			{
				cTemp = seqFile.get();
				if (cTemp == '\n')
				{
					break;
				}
			}
			// reading the sequence
			for (j = 0; j < sq.numSNP; j++)
			{
				if (DEBUG != 2)
				{
					cout << ".";
				}
				cTemp = seqFile.get();
				if (cTemp == '0' || cTemp == '1' || cTemp == '2' || cTemp == '?')
				{
					sq.seq[Ind2(i, j, sq.numSNP)] = cTemp;
				}
				else if (cTemp == '\n')
				{
					j--;
				}
				else
				{
					cout << "There are errors in the sequence file!" << endl;
					exit(0);
				}
			}
			if (DEBUG != 2)
			{
				cout << endl;
			}
		}
	}
	// check sum
	if (i != sq.numSamp - 1 || j != sq.numSNP)
	{
		cout << "The number of records in the sequence file is not right!" << endl;
		exit(0);
	}
	cout << "OK!  ";
	cout << sq.numSamp << " sequences read, each of the length " << sq.numSNP << "." << endl;

	// output sequences read
	if (DEBUG == 1)
	{
		for (i = 0; i < sq.numSamp; i++)
		{
			for (j = 0; j < sq.numSNP; j++)
			{
				char cTemp = sq.seq[Ind2(i, j, sq.numSNP)];
				cout << cTemp;
			}
			cout << endl;
		}
	}
}

//////////////////////////////////////////////////////////////////////////
// read in location parameters
// set:
// numSNP
void ReadLocsPara(ifstream& locsFile, Loci& lc)
{
	cout << "Reading locs parameters... ";

	double dTemp;
	char cTemp;
	
	locsFile >> lc.numSNP;

	locsFile >> dTemp >> cTemp;
	if (cTemp != 'L')
	{
		cout << "Sorry, this program could not be used to crossover model!" << endl;
		exit(0);
	}
	cout << "OK!" << endl;
}

//////////////////////////////////////////////////////////////////////////
// read in location file
// set:
// locs
void ReadLocs(ifstream& locsFile, Loci& lc)
{
	cout << "Reading SNP locations... ";
	int i = 0;
	while (!locsFile.eof())
	{
		locsFile >> lc.locs[i];
		i++;
		if (i >= lc.numSNP)
		{
			break;
		}
	}
	if (i != lc.numSNP)
	{
		cout << "Incomplete locs file!" << endl;
		exit(0);
	}

	cout << "OK!  "
		 << lc.numSNP << " SNP locations read." << endl;

	if (DEBUG == 1)
	{
		for (i = 0; i < lc.numSNP; i++)
		{
			cout << lc.locs[i] << "\t";
		}
		cout << endl;
	}
}

//////////////////////////////////////////////////////////////////////////
// judge whether a SNP should be selected
int JudgeSites(IntVect& iv, Sequ& sq, Loci& lc, double MAF)
{
	cout << "Calculate the MAF of each sites..." << endl;

	int i, j;

	int numSel = 0;

	for (i = 0; i < sq.numSNP; i++)
	{
		int freq0 = 0;
		int freq1 = 0;

		//////////////////////////////////////////////////////////////////////////
		// stat
		for (j = 0; j < sq.numSamp; j++)
		{
			switch (sq.seq[Ind2(j, i, sq.numSNP)])
			{
			case '0':
				freq0 += 2;
				break;
			case '1':
				freq1 += 2;
				break;
			case '2':
				freq0 += 1;
				freq1 += 1;
				break;
			default:
				;
			}
		}

		//////////////////////////////////////////////////////////////////////////
		// judge
		if (freq0 <= int(sq.numSamp * 2.0 * MAF) || freq1 <= int(sq.numSamp * 2.0 * MAF))
		{
			iv.vect[i] = 0;
		}
		else
		{
			iv.vect[i] = 1;
		}

		numSel += iv.vect[i];
	}

	cout << "OK! " << numSel << " sites with MAF > " << MAF << "." << endl;

	return numSel;
}

//////////////////////////////////////////////////////////////////////////
// select SNPs for sequences
void SelSNPSeq(Sequ& nsq, Sequ& sq, IntVect& iv)
{
	cout << "Select SNP sequences..." << endl;

	int i, j;
	for (i = 0; i < sq.numSamp; i++)
	{
		int nc = 0;
		for (j = 0; j < sq.numSNP; j++)
		{
			if (iv.vect[j] == 1)
			{
				nsq.seq[Ind2(i, nc, nsq.numSNP)] = sq.seq[Ind2(i, j, sq.numSNP)];
				nc++;
			}
		}

		//////////////////////////////////////////////////////////////////////////
		// chech whether right
		if (nc != nsq.numSNP)
		{
			cout << "Wrong selection!" << endl;
			exit(0);
		}
	}

	cout << "OK!" << endl;
	return;
}

//////////////////////////////////////////////////////////////////////////
// select SNPs for locations
void SelSNPLocs(Loci& nlc, Loci& lc, IntVect& iv)
{
	cout << "Select SNP locations..." << endl;

	int i;
	int nc = 0;
	for (i = 0; i < lc.numSNP; i++)
	{
		if (iv.vect[i] == 1)
		{
			nlc.locs[nc] = lc.locs[i];
			nc++;
		}
	}

	//////////////////////////////////////////////////////////////////////////
	// check whether right
	if (nc != nlc.numSNP)
	{
		cout << "Wrong selection!" << endl;
		exit(0);
	}

	cout << "OK!" << endl;
	return;
}

//////////////////////////////////////////////////////////////////////////
// write result file
void WriteRes(FileNames& fn, Loci& lc, RateSearch& rs, HotReg& hr, WinWeight& ww)
{
	//////////////////////////////////////////////////////////////////////////
	// write results to files
	ofstream resFile(fn.outFileName);
//	if (resFile.fail())
//	{
//		cout << "Error opening file " << fn.outFileName << ".!" << endl;
//		exit(0);
//	}
	resFile << "0" << "\t"
			<< setiosflags(ios::fixed) << setprecision(3) << lc.locs[0] << "\t"
			<< setiosflags(ios::fixed) << setprecision(3) << lc.locs[lc.numSNP-1] << "\t"
			<< setiosflags(ios::fixed) << setprecision(3) << rs.rate[hr.bkrate[hr.numHot-1]] << "\t"
			<< setiosflags(ios::fixed) << setprecision(3) << 0.0; // << "\t"
			//<< lc.numSNP << "\t"
			//<< setiosflags(ios::fixed) << setprecision(3) << lc.locs[0] << "\t"
			//<< setiosflags(ios::fixed) << setprecision(3) << lc.locs[lc.numSNP-1] << "\t"
			//<< 0;
	for (int ni = 0; ni < hr.numHot; ni++)
	{
		resFile << endl
				<< ni + 1 << "\t"
				<< setiosflags(ios::fixed) << setprecision(3) << lc.locs[0] + hr.hInter[ni].st * ww.grid << "\t"
				<< setiosflags(ios::fixed) << setprecision(3) << lc.locs[0] + hr.hInter[ni].ed * ww.grid << "\t"
				<< setiosflags(ios::fixed) << setprecision(3) << rs.rate[hr.hotrate[ni]] << "\t"
				<< setiosflags(ios::fixed) << setprecision(3) << hr.dLik[ni]; // << "\t"
				//<< hr.sint[ni].ed - hr.sint[ni].st << "\t"
				//<< setiosflags(ios::fixed) << setprecision(3) << lc.locs[hr.sint[ni].st] << "\t"
				//<< setiosflags(ios::fixed) << setprecision(3) << lc.locs[hr.sint[ni].ed + 1] << "\t"
				//<< hr.edgecor[ni];
	}
	return;
}
