#include <iostream.h>
#include <stdlib.h>
#include "regliktool.h"
#include "fisher.h"

//////////////////////////////////////////////////////////////////////////
// sum the recombination rate of two SNPs when the recombination rate is continued
// Note: No limitation about the rate returned.
double SumRate2SNP(double *aRate, Inter& sinter)
{
	//////////////////////////////////////////////////////////////////////////
	// to ensure that ed >= st
	if (sinter.st > sinter.ed)
	{
		int iTemp = sinter.st;
		sinter.st = sinter.ed;
		sinter.ed = iTemp;
	}
	//////////////////////////////////////////////////////////////////////////
	// sum the recombination rate
	double recomb = 0.0;
	int i;
	for (i = sinter.st; i < sinter.ed; i++)
	{
		recomb += aRate[i];
	}
	return recomb;
}

//////////////////////////////////////////////////////////////////////////
// get the likelihood when the recombination is not exact the value in the list
// Note: when the recombination rate is more than the maximum rate in the table, 
//		 return the likelihood of the maximum rate.
double LikCont(double aRate, double *likVect, LikTable& lt)
{
	//////////////////////////////////////////////////////////////////////////
	// check the minimum rate
	if (aRate < 0.0)
	{
		return likVect[0];
	}
	//////////////////////////////////////////////////////////////////////////
	// check the maximum rate
	if (aRate > lt.recombRate[lt.numRate-1])
	{
		return likVect[lt.numRate-1];
	}
	//////////////////////////////////////////////////////////////////////////
	// otherwise, one-order insider
	double step = lt.recombRate[lt.numRate-1] / (lt.numRate-1.0);
	int i = int(aRate / step);
	double lik = likVect[i] + (likVect[i+1] - likVect[i]) * (aRate / step - i);
	return lik;
}


//////////////////////////////////////////////////////////////////////////
// calculate the composite likelihood of the region between two SNPs
double RegLik(double *aRate, Inter& sinter, DataLikMatrix& lm, LikTable& lt)
{
	int i, j;
	double lik = 0.0;
	for (i = sinter.st; i <= sinter.ed-1; i++)
	{
		for (j = i+1; j <= sinter.ed; j++)
		{
			Inter nsint = {i, j};
			double rate2SNP = SumRate2SNP(aRate, nsint);
			double *likVect;
			likVect = lm.likMatrix + lm.typInd[angInd2(i, j, lm.numSNP)] * lm.numRate;
			lik += LikCont(rate2SNP, likVect, lt);
		}
	}
	return lik;
}

//////////////////////////////////////////////////////////////////////////
// composite likelihood of the whole region
double WholeRegLik(double *aRate, DataLikMatrix& lm, LikTable& lt)
{
	Inter sinter = {0, (lm.numSNP - 1)};
	return RegLik(aRate, sinter, lm, lt);
}

//////////////////////////////////////////////////////////////////////////
// average recombination rate index that maximize the composite likelihood between two SNPs
int AverRegRate(Inter& sinter, RateSearch& rs, Loci& lc, DataLikMatrix& lm, LikTable& lt)
{
	int i;
	int j;
	
	double lik;
	double maxLik;
	int maxInd;

	double *aRate = new double[lm.numSNP-1];	// aculmulated rate between each two continuous SNPs
	// search each rate in the search rate table
	for (i = 0; i < rs.nRate; i++)
	{
		//////////////////////////////////////////////////////////////////////////
		// initialize the recombination rates
		for (j = 0; j < lm.numSNP-1; j++)
		{
			aRate[j] = rs.rate[i] * (lc.locs[j+1] - lc.locs[j]);
		}
		//////////////////////////////////////////////////////////////////////////
		// calculate the composite likelihood between SNP iS & jS
		lik = RegLik(aRate, sinter, lm, lt);
		if (i == 0 || lik > maxLik)
		{
			maxLik = lik;
			maxInd = i;
		}
	}
	delete[] aRate;
	return maxInd;
}

//////////////////////////////////////////////////////////////////////////
// average recombination rate index that maximize the composite likelihood between two SNPs
int AverWholeRegRate(RateSearch& rs, Loci& lc, DataLikMatrix& lm, LikTable& lt)
{
	Inter sinter = {0, (lm.numSNP - 1)};
	return AverRegRate(sinter, rs, lc, lm, lt);
}
