#include <iostream.h>
#include <fstream.h>
#include <iomanip.h>
#include <stdlib.h>
#include <math.h>

#include "singlelik.h"
#include "vectool.h"
#include "fisher.h"

//////////////////////////////////////////////////////////////////////////
// get the pair genotype frequency
// here, input parameter 'numInd' should equal to number of sequences, i.e. number of genotypes that are counted.
void GetPairFreqGeno(int *freq, int *seq, int numSNP, int *ind, int numInd, int S1, int S2)
{
	int i;
	// initialization
	for (i = 0; i < 16; i++)
	{
		freq[i] = 0;
	}

	// calculate the frequencies
	int iS1, iS2;
	for (i = 0; i < numInd; i++)
	{
		switch (seq[Ind2(ind[i], S1, numSNP)])
		{
			case '?':
				{
					iS1 = 0;
					break;
				}
			case '0':
				{
					iS1 = 1;
					break;
				}
			case '1':
				{
					iS1 = 2;
					break;
				}
			default:	iS1 = 3;
		}
		switch (seq[Ind2(ind[i], S2, numSNP)])
		{
			case '?':
				{
					iS2 = 0;
					break;
				}
			case '0':
				{
					iS2 = 1;
					break;
				}
			case '1':
				{
					iS2 = 2;
					break;
				}
			default:	iS2 = 3;
		}
		freq[4 * iS1 + iS2]++;
	}

	// output genotype pair frequencies
	if (DEBUG == 1)
	{
		cout << endl << "Genotype pair frequencies:" << endl;
		for (i = 0; i < 16; i++)
		{
			cout << freq[i] << ' ';
		}
	}
}

//////////////////////////////////////////////////////////////////////////
// get the pair haplotype frequency
// here, input parameter 'numInd' should equal to number of sequences, i.e. number of haplotypes that are counted.
void GetPairFreqHap(int *freq, int *seq, int numSNP, int *ind, int numInd, int S1, int S2)
{
	int i;
	// initialization
	for (i = 0; i < 9; i++)
	{
		freq[i] = 0;
	}

	// calculate the frequencies
	for (i = 0; i < numInd; i++)
	{
		switch (seq[Ind2(ind[i], S1, numSNP)])
		{
		case '0':
			{
				switch (seq[Ind2(ind[i], S2, numSNP)])
				{
				case '0':
					{
						freq[0]++;
						break;
					}
				case '1':
					{
						freq[2]++;
						break;
					}
				default:
					{
						freq[4]++;
					}
				}
				break;
			}
		case '1':
			{
				switch (seq[Ind2(ind[i], S2, numSNP)])
				{
				case '0':
					{
						freq[1]++;
						break;
					}
				case '1':
					{
						freq[3]++;
						break;
					}
				default:
					{
						freq[5]++;
					}
				}
				break;
			}
		default:
			{
				switch (seq[Ind2(ind[i], S2, numSNP)])
				{
				case '0':
					{
						freq[6]++;
						break;
					}
				case '1':
					{
						freq[7]++;
						break;
					}
				default:
					{
						freq[8]++;
					}
				}
			}
		}
	}

	// output genotype pair frequencies
	if (DEBUG == 1)
	{
		cout << "Haplotype pair frequencies:" << endl;
		for (i = 0; i < 9; i++)
		{
			cout << freq[i] << ' ';
		}
		cout << endl;
	}
}

//////////////////////////////////////////////////////////////////////////
//	Calculate likelihoods for diploid data (with or without missing data)
//
// lknew: final likelihood ratio vector, output
// lkres: temp likelihood ratio vector
// freq: genotype pair frequencies, length-16 vector, main input
// lnfac: ln(n!) vector, length = 2 * numDip, input
// numDip: number of diploid sample, input
// numRate: number of different recombination rate, input
// lik:	originial haplotype likelihood, length = numComb * numRate, input
void CalDipLik(double *lknew, double *lkres, int *freq, double *lnfac, int numDip, int numRate, double *lik)
{
	int i, j;

	for (i = 0; i < numRate; i++)
	{
		lkres[i] = 0.0;		// likelihood in each possible combination
		lknew[i] = 0.0;		// final likelihood
	}

	int pbase[9];	// haplotype pair frequencies, maybe with missing data
	// Please note: in the next 9 lines, no proceeding about freq[15]
	pbase[0] = 2 * freq[5] + freq[7] + freq[13];
	pbase[1] = 2 * freq[9] + freq[11] + freq[13];
	pbase[2] = 2 * freq[6] + freq[7] + freq[14];
	pbase[3] = 2 * freq[10] + freq[11] + freq[14];
	pbase[4] = 2 * freq[4] + freq[12];
	pbase[5] = 2 * freq[8] + freq[12];
	pbase[6] = 2 * freq[1] + freq[3];
	pbase[7] = 2 * freq[2] + freq[3];
	pbase[8] = 2 * freq[0];
	int fl = pbase[4] + pbase[5] + pbase[6] + pbase[7] + pbase[8];	// fl = 0: no missing data; else, there is

	int tres[9];
	for (i = 0; i <= freq[15]; i++)		// different possibilities about 22 (perhaps 11/00, or perhaps 01/10)
	{
		double mn = lnfac[freq[15]] - lnfac[i] - lnfac[freq[15]-i];
		for (j = 0; j < 9; j++)
		{
			tres[j] = pbase[j];
		}
		tres[0] += i;
		tres[3] += i;
		tres[1] += freq[15]-i;
		tres[2] += freq[15]-i;
		OrderPtHap(tres, numDip * 2);

		if (DEBUG == 1)
		{
			cout << endl << "Res: ";
			for (j = 0; j < 9; j++)
			{
				cout << tres[j] << " ";
			}
		}

		if (fl)	// there is missing data
		{
			CalHapLikMis(lkres, tres, lnfac, numRate, numDip*2, lik);
		}
		else	// no missing data
		{
			// in fact, here tres is a length-4 vector
			// tres[0]: 00, tres[1]: 10, tres[2]: 01, tres[3]: 11, tres[4~8] = 0
			int p1 = tres[1] + tres[3];	// number of '1' in the first SNP
			int p2 = tres[2] + tres[3];	// number of '1' in the second SNP
			int hap = int(p1*(p1-1)*(p1+4)/6 + (p2-1)*(p2+2)/2 + tres[2]);	// the line in the original likelihood file

			if (DEBUG == 1)
			{
				cout << "\t\tRate table Index:  " << hap;
			}

			for (j = 0; j < numRate; j++)
			{
				lkres[j] = lik[Ind2(hap, j, numRate)];
			}
		}
		if (i == 0)
		{
			for (j = 0; j < numRate; j++)
			{
				lknew[j] = lkres[j];
			}
		}
		else
		{
			for (j = 0; j < numRate; j++)
			{
				lknew[j] += (double) log(1.0 + exp(mn + lkres[j] - lknew[j]));
			}
		}
	}

	if (DEBUG == 1)
	{
		cout << endl << "Summed likelihood: ";
		for (i = 0; i < numRate; i++)
		{
			cout << lknew[i] << " ";
		}
		cout << endl << endl;
	}
}


//////////////////////////////////////////////////////////////////////////
//	Calculate likelihoods for haploid data (with or without missing data)
//
// lknew: final likelihood ratio vector, output
// freq: genotype pair frequencies, length-16 vector, main input
// lnfac: ln(n!) vector, length = 2 * numDip, input
// numHap: number of haploid sample, input
// numRate: number of different recombination rate, input
// lik:	originial haplotype likelihood, length = numComb * numRate, input
void CalHapLik(double *lknew, int *freq, double *lnfac, int numHap, int numRate, double *lik)
{
	int i, j;

	int tres[9];
	for (i = 0; i < 9; i++)
	{
		tres[i] = freq[i];
	}

	// output haplotype
	if (DEBUG == 1)
	{
		cout << endl << "Res: ";
		for (j = 0; j < 9; j++)
		{
			cout << tres[j] << " ";
		}
	}

	OrderPtHap(tres, numHap);

	// output haplotype ordered
	if (DEBUG == 1)
	{
		cout << endl << "Res ordered: ";
		for (j = 0; j < 9; j++)
		{
			cout << tres[j] << " ";
		}
	}

	// initialization
	for (i = 0; i < numRate; i++)
	{
		lknew[i] = 0.0;		// likelihood in each possible combination
	}

	int fl = tres[4] + tres[5] + tres[6] + tres[7] + tres[8];	// fl = 0: no missing data; else, there is
	if (fl)	// there is missing data
	{
		CalHapLikMis(lknew, tres, lnfac, numRate, numHap, lik);
	}
	else	// no missing data
	{
		int p1 = tres[1] + tres[3];	// number of '1' in the first SNP
		int p2 = tres[2] + tres[3];	// number of '1' in the second SNP
		int hap = int(p1*(p1-1)*(p1+4)/6 + (p2-1)*(p2+2)/2 + tres[2]);	// the line in the original likelihood file
		
		if (DEBUG == 1)
		{
			cout << "\t\tRate table index:  " << hap;
		}
		
		for (j = 0; j < numRate; j++)
		{
			lknew[j] = lik[Ind2(hap, j, numRate)];
		}
	}

	if (DEBUG == 1)
	{
		cout << endl << "Summed likelihood: ";
		for (i = 0; i < numRate; i++)
		{
			cout << lknew[i] << " ";
		}
		cout << endl << endl;
	}
}

//////////////////////////////////////////////////////////////////////////
// order haplotype
void OrderPtHap(int *pt, int nseq) 
{
	int fl=0;

	if (2 * (pt[1]+pt[3]+pt[5]) > nseq)
	{
		fl += 2;
	}
	if (2 * (pt[2]+pt[3]+pt[7]) > nseq)
	{
		fl += 1;
	}
	
	switch (fl)
	{
	case 0:
		{
			break;
		}
	case 1:
		{	// RHS only
			SwapPt(pt, 0, 2);
			SwapPt(pt, 1, 3);
			SwapPt(pt, 6, 7);
			break;
		}
	case 2:
		{	// LHS only
			SwapPt(pt, 0, 1);
			SwapPt(pt, 2, 3);
			SwapPt(pt, 4, 5);
			break;
		}
	case 3:
		{	// Both sides
			SwapPt(pt, 0, 3);
			SwapPt(pt, 1, 2);
			SwapPt(pt, 4, 5);
			SwapPt(pt, 6, 7);
			break;
		}
	default:
		{
			cout << "Error in sorting!" << endl;
			exit(0);
		}
	}

	// Next Line must commented out if use multiple mutation rates

	if (pt[2]+pt[7] > pt[1]+pt[5])
	{
		SwapPt(pt, 1, 2);
		SwapPt(pt, 5, 7);
		SwapPt(pt, 4, 6);
	}
}

//////////////////////////////////////////////////////////////////////////
// exchange two elements of a vector
void SwapPt(int *pt, int s1, int s2) 
{     
	int temp = pt[s2];
	pt[s2] = pt[s1];
	pt[s1] = temp; 
}


//////////////////////////////////////////////////////////////////////////
//	Calculate haploid likelihoods for missing data
// lkmiss: final likelihood ratio vector, output
// pt: haplotype pair frequencies, length-9 vector, main input
// lnfac: ln(n!) vector, length = 2 * numDip, input
// numDip: number of diploid sample, input
// numRate: number of different recombination rate, input
// lik:	originial haplotype likelihood, length = numComb * numRate, input
void CalHapLikMis(double *lkmiss, int *pt, double *lnfac, int numRate, int numHap, double *lik)
{
	int i;
	int k;

	//////////////////////////////////////////////////////////////////////////
	// output haplotype
	if (DEBUG == 1)
	{
		cout << endl << "Hap: ";
		for (i = 0; i < 9; i++)
		{
			cout << "|" << pt[i] << "|";
		}
	}

	//////////////////////////////////////////////////////////////////////////
	//	initialization
	for (i = 0; i < numRate; i++)
	{
		lkmiss[i] = 0.0;
	}
	
	int pres[9];
	for (i = 0; i < 9; i++)
	{
		pres[i] = 0;
	}

	int ct = 0;
	for (int a = 0; a <= pt[4]; a++)
	{
		for (int b = 0; b <= pt[5]; b++)
		{
			for (int c = 0; c <= pt[6]; c++)
			{
				for (int d = 0; d <= pt[7]; d++)
				{
					for (int e1 = 0; e1 <= pt[8]; e1++)
					{
						for (int e2 = 0; e2 <= pt[8]-e1; e2++)
						{
							for (int e3 = 0; e3 <= pt[8]-e1-e2; e3++)
							{
								ct++;
								int e4 = pt[8]-e1-e2-e3;
								pres[0] = pt[0] + a + c + e1;				// 00
								pres[1] = pt[1] + b + pt[6]-c + e2;			// 10
								pres[2] = pt[2] + pt[4]-a + d + e3;			// 01
								pres[3] = pt[3] + pt[5]-b + pt[7]-d + e4;	// 11
								// pres[4~8] = 0

								OrderPtHap(pres, numHap);
								
								int p1 = pres[1] + pres[3];
								int p2 = pres[2] + pres[3];
								int ht = int(p1*(p1-1)*(p1+4)/6 + (p2-1)*(p2+2)/2 + pres[2]);

								if (DEBUG == 1)
								{
									cout << "\t\tRate table index:  " << ht;
								}

								if (DEBUG == 1)
								{
									cout << endl << "Res " << ct << ": ht " << ht << ": ";
									for (i = 0; i < 4; i++)
									{
										cout << "|" << pres[i] << "|";
									}
								}
								
								double mn = lnfac[pt[4]] - lnfac[a] - lnfac[pt[4]-a];
								mn += lnfac[pt[5]] - lnfac[b] - lnfac[pt[5]-b];
								mn += lnfac[pt[6]] - lnfac[c] - lnfac[pt[6]-c];
								mn += lnfac[pt[7]] - lnfac[d] - lnfac[pt[7]-d];
								mn += lnfac[pt[8]] - lnfac[e1] - lnfac[e2] - lnfac[e3] - lnfac[e4];

								if (DEBUG == 1)
								{
									cout << endl << "log(mn coff) = " << mn;
								}

								// Better routine for summation
								if (ct == 1)	// the first one
								{
									for (k = 0; k < numRate; k++)
									{
										lkmiss[k] = lik[Ind2(ht, k, numRate)];
									}
								}
								else
								{
									for (k = 0; k < numRate; k++)
									{
										lkmiss[k] += log(1.0 + exp(lik[Ind2(ht, k, numRate)] + mn - lkmiss[k]));	// equal to "exp(lkmiss[k]) += exp(lkmat[ht][k] + mn);"
									}
								}
							}
						}
					}
				}
			}
		}
	}

	if (DEBUG == 1)
	{
		cout << endl << "Lk : ";
		for (k = 0; k < numRate; k++)
		{
			cout << lkmiss[k] << " ";
		}
	}
}

//////////////////////////////////////////////////////////////////////////
// get the log(n!) vector
void GetLnfac(double *lnfac, int len)
{
	lnfac[0] = 0;	// ln(0!) = 0
	lnfac[1] = 0;	// ln(1!) = 0
	int i;
	for (i = 2; i < len; i++)
	{
		lnfac[i] = lnfac[i-1] + log(i + 0.0);	// ln(n!) = ln((n-1)!) + ln(n)
	}
}

//////////////////////////////////////////////////////////////////////////
// decide which SNPs have a minor allele frequencies less that thrd
// Especially,
// thrd = 0.0	--	only delete the homogeneous SNPs
// thrd = -1	--  no filtering
// Note: this routine could be applied to whether haplotype or genotype data
int SnpHetero(int *hetInd, int *homoInd, int *seq, int numSNP, int *ind, int numInd, double thrd)
{
	if (DEBUG != 2)
	{
		cout << "Decide which SNPs are homogeneous in the bootstrap samples ...";
	}
	int hetero = 0;
	int homo = 0;
	int i, j, k;
	for (i = 0; i < numSNP; i++)
	{
		int majA = 0;	// the number of major allele
		int minA = 0;	// the number of minor allele
		for (j = 0; j < numInd; j++)
		{
			switch (seq[Ind2(ind[j], i, numSNP)])
			{
			case '0':
				{
					majA += 2;
					break;
				}
			case '1':
				{
					minA += 2;
					break;
				}
			case '2':
				{
					majA++;
					minA++;
					break;
				}
			default:
				;
			}
		}

		if (majA > int(2 * numInd * thrd) && minA > int(2 * numInd * thrd))
		{
			hetInd[hetero] = i;
			hetero++;
		}
		else
		{
			homoInd[homo] = i;
			homo++;
		}
	}
	if (DEBUG != 2)
	{
		cout << "OK!";

		if (homo == 0)
		{
			cout << endl << "No SNPs are filtered." << endl;
		}
		else
		{
			cout << endl << "The SNPs that are filtered are: ";
			for (k = 0; k < homo; k++)
			{
				cout << homoInd[k] << "  ";
			}
			cout << endl;
		}
	}
	
	return hetero;
}

//////////////////////////////////////////////////////////////////////////
// output likelihood file
void OutLikFile(DataLikMatrix& lm, int procI, int numRate, int hg)
{
	int numFREQ = 9;
	if (hg == 2)
	{
		numFREQ = 16;
	}
	cout << "Write likelihood matrix to file outlike.txt... ";
	ofstream outLikHandle("outlik.txt");
//	if (outLikHandle.fail())
//	{
//		cerr << "Cannot open likelihood file to output!" << endl;
//		exit(0);
//	}
	int i, j;
	for (i = 0; i < procI; i++)
	{
		outLikHandle << setiosflags(ios::fixed) << setw(6) << setprecision(1) <<  lm.typInd[i] << ":";
		for (j = 0; j < numFREQ; j++)
		{
			outLikHandle << "\t" << setiosflags(ios::fixed) << setw(3) << setprecision(1) << lm.genoInd[Ind2(i, j, numFREQ)];
		}
		outLikHandle << ":";
		for (j = 0; j < numRate; j++)
		{
			outLikHandle << "\t" << setiosflags(ios::fixed) << setw(9) << setprecision(2) << lm.likMatrix[Ind2(i, j, numRate)];
		}
		outLikHandle << endl;
	}
	cout << "OK!" << endl;
}
