#include <iostream.h>
#include <fstream.h>
#include <stdlib.h>
#include <iomanip.h>

#include "fisher.h"
#include "truncliktool.h"
#include "regliktool.h"
#include "vectool.h"

//////////////////////////////////////////////////////////////////////////
// calculate the composite likelihood of the region between two SNPs
double TruncRegLik(double *aRate, Inter& sinter, DataLikMatrix& lm, LikTable& lt, WinWeight& ww)
{
	int i, j;
	int d;	// delta
	double lik = 0.0;
	for (i = sinter.st; i <= sinter.ed-1; i++)
	{
		for (d = 1; d <= ww.len; d++)
		{
			j = i + d;
			if (j > sinter.ed)
			{
				break;
			}
			Inter nsint = {i, j};
			double rate2SNP = SumRate2SNP(aRate, nsint);
			double *likVect;
			likVect = lm.likMatrix + lm.typInd[angInd2(i, j, lm.numSNP)] * lm.numRate;
			lik += LikCont(rate2SNP, likVect, lt) * ww.wt[d-1];
		}
	}
	return lik;
}

//////////////////////////////////////////////////////////////////////////
// composite likelihood of the whole region
double TruncWholeRegLik(double *aRate, DataLikMatrix& lm, LikTable& lt, WinWeight& ww)
{
	Inter sinter = {0, (lm.numSNP - 1)};
	return TruncRegLik(aRate, sinter, lm, lt, ww);
}

//////////////////////////////////////////////////////////////////////////
// average recombination rate index that maximize the composite likelihood between two SNPs
int TruncAverRegRate(Inter& sinter, RateSearch& rs, Loci& lc, DataLikMatrix& lm, LikTable& lt, WinWeight& ww)
{
	int i;
	int j;
	
	double lik;
	double maxLik;
	int maxInd;

	double *aRate = new double[lm.numSNP-1];	// aculmulated rate between each two continuous SNPs
	// search each rate in the search rate table
	for (i = 0; i < rs.nRate; i++)
	{
		//////////////////////////////////////////////////////////////////////////
		// initialize the recombination rates
		for (j = 0; j < lm.numSNP-1; j++)
		{
			aRate[j] = rs.rate[i] * (lc.locs[j+1] - lc.locs[j]);
		}
		//////////////////////////////////////////////////////////////////////////
		// calculate the composite likelihood between SNP iS & jS
		lik = TruncRegLik(aRate, sinter, lm, lt, ww);
		if (i == 0 || lik > maxLik)
		{
			maxLik = lik;
			maxInd = i;
		}
	}
	delete[] aRate;
	return maxInd;
}

//////////////////////////////////////////////////////////////////////////
// average recombination rate index that maximize the composite likelihood between two SNPs
int TruncAverWholeRegRate(RateSearch& rs, Loci& lc, DataLikMatrix& lm, LikTable& lt, WinWeight& ww)
{
	Inter sinter = {0, (lm.numSNP - 1)};
	return TruncAverRegRate(sinter, rs, lc, lm, lt, ww);
}

//////////////////////////////////////////////////////////////////////////
// convert the rates of grids to the accumulated rates between SNPs
void GridRate2SNPaRate(GridRate& gr, SNPRate& sr, RateSearch& rs)
{
	int i, j;
	for (i = 0; i < sr.numSNP-1; i++)
	{
		sr.aRate[i] = 0.0;
		//////////////////////////////////////////////////////////////////////////
		// the grid inter that overlap with the SNP inter
		Inter gpinter;
		gpinter.st = int((sr.locs[i] - gr.pos[0]) / gr.grid);		// the start index of the first grid gap
		gpinter.ed = int((sr.locs[i+1] - gr.pos[0]) / gr.grid);	// the start index of the last grid gap
		//// to avoid out of vector length
		if (gr.pos[0] + gpinter.ed * gr.grid > sr.locs[i+1] - 0.0005)
		{
			gpinter.ed -= 1;
		}

		//////////////////////////////////////////////////////////////////////////
		// sum the recombination rate of the inter
		if (gpinter.st == gpinter.ed)
		{
			sr.aRate[i] += rs.rate[gr.iRate[gpinter.st]] * (sr.locs[i+1] - sr.locs[i]);
		}
		else
		{
			sr.aRate[i] += rs.rate[gr.iRate[gpinter.st]] * (gr.pos[gpinter.st + 1] - sr.locs[i]);
			for (j = gpinter.st+1; j < gpinter.ed; j++)
			{
				sr.aRate[i] += rs.rate[gr.iRate[j]] * gr.grid;
			}
			sr.aRate[i] += rs.rate[gr.iRate[gpinter.ed]] * (sr.locs[i+1] - gr.pos[gpinter.ed]);
		}
	}
}

//////////////////////////////////////////////////////////////////////////
// calculate background rate that maximize truncated pairwise likelihood
int TruncMaxBkRate(HotReg& hr, GridRate& gr, SNPRate& sr, RateSearch& rs, DataLikMatrix& lm, LikTable& lt, WinWeight& ww, int outFlag)
{
	int i, j, k;
	//////////////////////////////////////////////////////////////////////////
	// output background tags
	if (DEBUG != 2 && outFlag != 0)
	{
		cout << "The background tags are:" << endl;
		for (i = 0; i < gr.numGrid-1; i++)
		{
			cout << "\t" << hr.back[i];
		}
		cout << endl;
	}

	//////////////////////////////////////////////////////////////////////////
	// get the background grid blocks
	Inter *gint = new Inter[gr.numGrid-1];	// backgound inter
	int numBlock = 0;
	for (i = 0; i < gr.numGrid-1; i++)
	{
		//// the first grid
		if (i == 0 && hr.back[i] == 0)
		{
			gint[numBlock].st = 0;
		}
		//// others
		if (i != 0)
		{
			if (hr.back[i] == 0 && hr.back[i-1] == 1)
			{
				gint[numBlock].st = i;
			}
			else if (hr.back[i] == 1 && hr.back[i-1] == 0)
			{
				gint[numBlock].ed = i;
				numBlock++;
			}
		}
		//// the last grid 
		if (i == gr.numGrid - 2 && hr.back[i] == 0)
		{
			gint[numBlock].ed = gr.numGrid - 1;
			numBlock++;
		}
	}
	///// output it
	if (DEBUG != 2 && outFlag != 0)
	{
		cout << numBlock << " background blocks." << endl
			 << "Their grid startings and endings are:" << endl;
		for (i = 0; i < numBlock; i++)
		{
			cout << gint[i].st << "\t" << gint[i].ed << endl;
		}
		cout << endl;
	}

	//////////////////////////////////////////////////////////////////////////
	// get the background SNP blocks
	Inter *sint = new Inter[gr.numGrid-1];
	for (j = 0; j < numBlock; j++)
	{
		//// start
		for (i = 0; i < sr.numSNP; i++)
		{
			if (sr.locs[i] > gr.pos[gint[j].st] - 0.0005)
			{
				break;
			}
		}
		sint[j].st = i;
		//// end
		for (k = i; k < sr.numSNP; k++)
		{
			if (sr.locs[k] > gr.pos[gint[j].ed] + 0.0005)
			{
				break;
			}
		}
		sint[j].ed = k - 1;
		//// edge
		if (k == sr.numSNP-1 && sr.locs[k] > gr.pos[gint[j].ed] - 0.0005)
		{
			sint[j].ed = k;
		}
	}
	
	//// output
	if (DEBUG != 2 && outFlag != 0)
	{
		cout << numBlock << " background blocks." << endl
			 << "Their SNP startings and endings are:" << endl;
		for (i = 0; i < numBlock; i++)
		{
			cout << sint[i].st << "\t" << sint[i].ed << endl;
		}
	}

	//////////////////////////////////////////////////////////////////////////
	// begin calculation
	int maxBkRate;
	double maxBkLik;
	double bkLik;
	int bkrate;
	for (bkrate = 0; bkrate < rs.nRate; bkrate++)
	{
		//////////////////////////////////////////////////////////////////////////
		// set all the recombination rate to the trying bkrate
		for (i = 0; i < gr.numGrid-1; i++)
		{
			gr.iRate[i] = bkrate;
		}
		GridRate2SNPaRate(gr, sr, rs);

		bkLik = 0.0;
		//////////////////////////////////////////////////////////////////////////
		// calculate the likelihood
		for (i = 0; i < numBlock; i++)
		{
			if (sint[i].ed > sint[i].st)
			{
				bkLik += TruncRegLik(sr.aRate, sint[i], lm, lt, ww);
			}
		}
		if (DEBUG != 2 && outFlag != 0)
		{
			cout << "back ground rate = " << bkrate << "\t\t\t" << "likelihood = " << bkLik << endl;
		}

		//////////////////////////////////////////////////////////////////////////
		// update if necessary
		if (bkrate == 0 || bkLik > maxBkLik)
		{
			maxBkLik = bkLik;
			maxBkRate = bkrate;
		}
	}

	delete[] gint;
	delete[] sint;
	
	return maxBkRate;
}

//////////////////////////////////////////////////////////////////////////
// search the hotspot recombination rate that maximize the truncated pairwise likelihood of the whole region
int TruncMaxHotRate(Inter& hot, HotReg& hr, GridRate& gr, SNPRate& sr, RateSearch& rs, DataLikMatrix& lm, LikTable& lt, WinWeight& ww, double& maxLik, int outFlag)
{
	int i;

	//////////////////////////////////////////////////////////////////////////
	// set original recombination rate
	for (i = 0; i < gr.numGrid-1; i++)
	{
		gr.iRate[i] = hr.iRate[i];
	}

	//////////////////////////////////////////////////////////////////////////
	// get the background recombination rate
	int bkRate;
	for (i = 0; i < gr.numGrid - 1; i++)
	{
		if (hr.back[i] == 0)
		{
			bkRate = hr.iRate[i];
			break;
		}
	}

	//////////////////////////////////////////////////////////////////////////
	// output
	if (DEBUG != 2 && outFlag != 0)
	{
		cout << "Original recombination rate:" << endl;
		for (i = 0; i < gr.numGrid-1; i++)
		{
			cout << "\t" << gr.iRate[i];
		}
		cout << endl;
	}

	//////////////////////////////////////////////////////////////////////////
	// search hotspot rate
	int hotRate;
	int maxHotRate;
	double lik;
	for (hotRate = bkRate; hotRate < rs.nRate; hotRate++)		// Note: hotspot rate should be bigger than background rate
	{
		for (i = hot.st; i < hot.ed; i++)
		{
			gr.iRate[i] = hotRate;
		}

		GridRate2SNPaRate(gr, sr, rs);
		lik = TruncWholeRegLik(sr.aRate, lm, lt, ww);
		
		if (hotRate == bkRate || lik > maxLik)
		{
			maxLik = lik;
			maxHotRate = hotRate;
		}

		//////////////////////////////////////////////////////////////////////////
		// output
		if (DEBUG != 2 && outFlag != 0)
		{
			cout << "hotRate = " << hotRate << "\t\t" << "likelihood = " << lik << endl;
		}
	}
	return maxHotRate;
}

//////////////////////////////////////////////////////////////////////////
// get number of grids
void CalNumGrid(GridRate& gr, Loci& lc, WinWeight& ww)
{
	////// grid
	gr.grid = ww.grid;
	
	///// numGrid
	gr.numGrid = int((lc.locs[lc.numSNP-1] - lc.locs[0]) / gr.grid);
	if (lc.locs[lc.numSNP-1] > lc.locs[0] + gr.numGrid * gr.grid + 0.0005)
	{
		gr.numGrid += 2;
	}
	else
	{
		gr.numGrid += 1;
	}

	return;
}

//////////////////////////////////////////////////////////////////////////
// intialize grids
void IniGrid(GridRate& gr, Loci& lc, WinWeight& ww)
{
	cout << endl << "Initializing grid rate..." << endl;

	int i;
	///// pos
	for (i = 0; i < gr.numGrid; i++)
	{
		gr.pos[i] = lc.locs[0] + gr.grid * i;
	}
	
	cout << "OK!" << endl;

	//////////////////////////////////////////////////////////////////////////
	// out results
	cout << "Number of grid = " << gr.numGrid << ", each with length = " << gr.grid << endl;
	cout << "Their positions are from " << gr.pos[0] << " to " << gr.pos[gr.numGrid-1] << "." << endl;

	if (DEBUG != 2)
	{
		cout << endl;
		cout << "Their positions are:";
		for (i = 0; i < gr.numGrid; i++)
		{
			cout << "\t" << gr.pos[i];
		}
		cout << endl;
	}

	return;
}

//////////////////////////////////////////////////////////////////////////
// initialize SNP rate
void IniSNPRate(SNPRate& sr, Loci& lc)
{
	cout << endl << "Initializing SNP rate..." << endl;
	int i;

	for (i = 0; i < sr.numSNP; i++)
	{
		sr.locs[i] = lc.locs[i];
	}

	cout << "OK!" << endl;

	//////////////////////////////////////////////////////////////////////////
	// out results
	if (DEBUG != 2)
	{
		cout << endl;
		cout << "Number of SNPs = " << sr.numSNP << "." << endl;
		
		cout << "Their positions are:";
		for (i = 0; i < sr.numSNP; i++)
		{
			cout << "\t" << sr.locs[i];
		}
		cout << endl;
	}
}


//////////////////////////////////////////////////////////////////////////
// initialize hotspot information of the whole region
void IniHotReg(HotReg& hr, GridRate& gr, SNPRate& sr, RateSearch& rs, Loci& lc, DataLikMatrix& lm, LikTable& lt, WinWeight& ww)
{
	cout << "Initializing hot region..." << endl;

	int i;
	hr.numGrid = gr.numGrid;
	hr.back = new int[hr.numGrid-1];
	hr.free = new int[hr.numGrid-1];
	hr.hot = new int[hr.numGrid-1];
	hr.iRate = new int[hr.numGrid-1];
	hr.deltaLik = new double[hr.numGrid-1];

	//!!!!!!if we want to set different background rate, we only need to change "for" below.
	//////////////////////////////////////////////////////////////////////////
	// set background tags
	if (ww.bkAll == 1)
	{
		for (i = 0; i < hr.numGrid-1; i++)
		{
			hr.back[i] = 0;
		}
	}
	else
	{
 		SetBkTags(hr, gr, sr, rs, lc, lm, lt, ww);
	}
	
	//////////////////////////////////////////////////////////////////////////
	// set free tags
	for (i = 0; i < hr.numGrid-1; i++)
	{
		hr.free[i] = 0;
	}

	//////////////////////////////////////////////////////////////////////////
	// set hot tags
	for (i = 0; i < hr.numGrid-1; i++)
	{
		hr.hot[i] = 0;
	}

	//////////////////////////////////////////////////////////////////////////
	// set rate tags
	int backRate = TruncMaxBkRate(hr, gr, sr, rs, lm, lt, ww, 1);
	for (i = 0; i < hr.numGrid-1; i++)
	{
		hr.iRate[i] = backRate;
	}

	//////////////////////////////////////////////////////////////////////////
	// set delta likelihoods
	for (i = 0; i < hr.numGrid-1; i++)
	{
		hr.deltaLik[i] = 0.0;
	}

	cout << "OK!" << endl;

	//////////////////////////////////////////////////////////////////////////
	// set hotspot informations
	hr.numHot = 0;
	hr.hInter = new Inter[hr.numGrid-1];
	hr.dLik = new double[hr.numGrid-1];

	//////////////////////////////////////////////////////////////////////////
	// output
	if (DEBUG != 2)
	{
		cout << "Background tags are:" << endl;
		for (i = 0; i < hr.numGrid-1; i++)
		{
			cout << "\t" << hr.back[i];
		}
		cout << endl;
		cout << "Free to search tags are:" << endl;
		for (i = 0; i < hr.numGrid-1; i++)
		{
			cout << "\t" << hr.free[i];
		}
		cout << endl;
		cout << "Hot tags are:" << endl;
		for (i = 0; i < hr.numGrid-1; i++)
		{
			cout << "\t" << hr.hot[i];
		}
		cout << endl;
		cout << "Delta likelihoods are:" << endl;
		for (i = 0; i < hr.numGrid-1; i++)
		{
			cout << "\t" << hr.deltaLik[i];
		}
		cout << endl;
		cout << "Recombination rates are:" << endl;
		for (i = 0; i < hr.numGrid-1; i++)
		{
			cout << "\t" << hr.iRate[i];
		}
		cout << endl;
	}
}

//////////////////////////////////////////////////////////////////////////
// part of initialize background tags
void SetBkTags(HotReg& hr, GridRate& gr, SNPRate& sr, RateSearch& rs, Loci& lc, DataLikMatrix& lm, LikTable& lt, WinWeight& ww)
{
	int i, j;

	int bkRate = TruncAverWholeRegRate(rs, lc, lm, lt, ww);
	
	//////////////////////////////////////////////////////////////////////////
	// background measured in SNPs
	int *back = new int[lc.numSNP-1];
	for (i = 0; i < lc.numSNP-1; i++)
	{
		Inter sinter = {i, i+1};
		int iRate = AverRegRate(sinter, rs, lc, lm, lt);
		if (rs.rate[iRate] > ww.higher * rs.rate[bkRate])
		{
			back[i] = 1;
		}
		else
		{
			back[i] = 0;
		}
	}

	//////////////////////////////////////////////////////////////////////////
	// background measured in grids
	for (i = 0; i < hr.numGrid-1; i++)
	{
		hr.back[i] = 0;
	}
	Inter ginter;
	for (i = 0; i < lc.numSNP-1; i++)
	{
		if (back[i] == 1)
		{
			ginter.st = int((sr.locs[i] + 0.0005 - gr.pos[0]) / gr.grid);
			ginter.ed = int((sr.locs[i+1] - 0.0005 - gr.pos[0]) / gr.grid) + 1;
			for (j = ginter.st; j < ginter.ed; j++)
			{
				hr.back[j] = 1;
			}
		}
	}

	delete[] back;
}
//
////////////////////////////////////////////////////////////////////////////
////
////		main routine to search recombination hotspots
////
////////////////////////////////////////////////////////////////////////////
//void TruncHot(RateSearch& rs, Loci& lc, DataLikMatrix& lm, LikTable& lt, WinWeight& ww, HotReg& hr)
//{
//	cout << endl << "Searching recombination hotspots..." << endl;
//
//	int i;
//	//////////////////////////////////////////////////////////////////////////
//	// initialization
//	GridRate gr;
//	SNPRate sr;
//	IniGridRate(gr, lc, ww);
//	IniSNPRate(sr, lc);
//	IniHotReg(hr, gr, sr, rs, lc, lm, lt, ww);
//
//	//////////////////////////////////////////////////////////////////////////
//	// if the length of hotspot is bigger than the region length
//	if (ww.minHotLen > gr.numGrid - 1)
//	{
//		return;
//	}
//
//	//////////////////////////////////////////////////////////////////////////
//	// searching recombination rate
//	NewHot *nHot = new NewHot[(gr.numGrid-1) * (ww.maxHotLen-ww.minHotLen+1)];	// new hotspot
//	int pos;		// start position of the hotspot searched
//	int len;		// length of the hotspot searched
//
//	HotReg temphr;
//	temphr.numGrid = hr.numGrid;
//	temphr.back = new int[temphr.numGrid-1];
//	temphr.free = new int[temphr.numGrid-1];
//	temphr.hot = new int[temphr.numGrid-1];
//	temphr.iRate = new int[temphr.numGrid-1];
//
//	int iter;
//	for (iter = 0; iter < MAX_ITER; iter++)
//	{
//		if (DEBUG != 2)
//		{
//			cerr << endl << endl << "ITERATION " << iter << "." << endl;
//		}
//		else
//		{
//			cout << endl << endl << "ITERATION " << iter << "." << endl;
//		}
//
//		for (i = 0; i < gr.numGrid-1; i++)
//		{
//			gr.iRate[i] = hr.iRate[i];
//		}
//		GridRate2SNPaRate(gr, sr, rs);
//		double oriLik = TruncWholeRegLik(sr.aRate, lm, lt, ww);		// original likelihood
//		cout << "Original truncated pairwise likelihood = " << oriLik << "." << endl;
//		
//		//////////////////////////////////////////////////////////////////////////
//		// searching all possible hotspots
//		for (pos = 0; pos < gr.numGrid-1; pos++)
//		{
//			for (len = ww.maxHotLen; len >= ww.minHotLen; len--)
//			{
//				int iHot = Ind2(pos, len-ww.minHotLen, (ww.maxHotLen-ww.minHotLen+1));
//
//				nHot[iHot].gint.st = pos;
//				nHot[iHot].gint.ed = pos + len;
//				
//				if (DEBUG != 2)
//				{
//					cout << "Start grid = " << nHot[iHot].gint.st << "; End grid = " << nHot[iHot].gint.ed << ".";
//				}
//				//////////////////////////////////////////////////////////////////////////
//				// judge if the location of hotspot is permitted
//				if (nHot[iHot].gint.ed > gr.numGrid-1)
//				{
//					nHot[iHot].fl = 0;
//					if (DEBUG != 2)
//					{
//						cout << "\tNot available." << endl;
//					}
//					continue;
//				}
//				
//				int ifFree = 0;
//				for (i = nHot[iHot].gint.st; i < nHot[iHot].gint.ed; i++)
//				{
//					if (hr.free[i] != 0)
//					{
//						ifFree = 1;
//						break;
//					}
//				}
//				if (ifFree == 1)
//				{
//					nHot[iHot].fl = 0;
//					if (DEBUG != 2)
//					{
//						cout << "\tNot available." << endl;
//					}
//					continue;
//				}
//
//				nHot[iHot].fl = 1;
//				//////////////////////////////////////////////////////////////////////////
//				// calculate the most fitted model
//				//// set temp hotreg
//				for (i = 0; i < hr.numGrid-1; i++)
//				{
//					temphr.back[i] = hr.back[i];
//					temphr.free[i] = hr.free[i];
//					temphr.hot[i] = hr.hot[i];
//					temphr.iRate[i] = hr.iRate[i];
//				}
//				//// reset temp back
//				/////////!!!!! whether to set its neighborhood as none background is under discussion
//				for (i = nHot[iHot].gint.st; i < nHot[iHot].gint.ed; i++)
//				{
//					temphr.back[i] = 1;
//				}
//				/// background rate
//				nHot[iHot].bkRate = TruncMaxBkRate(temphr, gr, sr, rs, lm, lt, ww, 0);
//				for (i = 0; i < hr.numGrid-1; i++)
//				{
//					if (temphr.hot[i] == 0)
//					{
//						temphr.iRate[i] = nHot[iHot].bkRate;
//					}
//				}
//				/// hotspot rate
//				double lik;
//				nHot[iHot].hotRate = TruncMaxHotRate(nHot[iHot].gint, temphr, gr, sr, rs, lm, lt, ww, lik, 0);
//				//// delta likelihood
//				nHot[iHot].dLik = lik - oriLik;
//				//// do edge compensation
//				double mpos = (gr.pos[nHot[iHot].gint.st] + gr.pos[nHot[iHot].gint.ed]) / 2.0;
//				int si;
//				for (si = 1; si < sr.numSNP; si++)
//				{
//					if (mpos < sr.locs[si])
//					{
//						break;
//					}
//				}
//				if (si == sr.numSNP)
//				{
//					si -= 1;
//				}
//				si -= 1;		// the index of the gap
//				int lInd = si - 0 + 1;				// the index ("k", begin from 1) from the left
//				int rInd = sr.numSNP-2 - si + 1;	// the index ("k", begin from 1) from the right
//				int rdInd = lInd;
//				if (rdInd > rInd)
//				{
//					rdInd = rInd;
//				}
//				if (rdInd < ww.len)
//				{
//					nHot[iHot].dLik /= ww.ewt[rdInd-1];
//					if (DEBUG != 2)
//					{
//						cout << "\tEdge correction. edge index = " << rdInd << ".";
//					}
//				}
//				if (DEBUG != 2)
//				{
//					cout << endl;
//				}
//			}
//		}
//
//		//////////////////////////////////////////////////////////////////////////
//		// output delta likelihood matrix
//		if (DEBUG != 2)
//		{
//			cout << "delta likelihood matrix is:" << endl;
//			for (pos = 0; pos < gr.numGrid-1; pos++)
//			{
//				for (len = ww.minHotLen; len <= ww.maxHotLen; len++)
//				{
//					int iHot = Ind2(pos, len-ww.minHotLen, (ww.maxHotLen-ww.minHotLen+1));
//					if (nHot[iHot].fl == 0)
//					{
//						cout << "\t" << setiosflags(ios::fixed) << setw(8) << setprecision(1) << -1.0;
//					}
//					else
//					{
//						cout << "\t" << setiosflags(ios::fixed) << setw(8) << setprecision(1) << nHot[iHot].dLik;
//					}
//				}
//				cout << endl;
//			}
//		}
//
//		//////////////////////////////////////////////////////////////////////////
//		// find the maximal delta likelihood
//		int maxI = 0;
//		double maxdLik = 0;
//		for (pos = 0; pos < gr.numGrid-1; pos++)
//		{
//			for (len = ww.maxHotLen; len >= ww.minHotLen; len--)
//			{
//				int iHot = Ind2(pos, len-ww.minHotLen, (ww.maxHotLen-ww.minHotLen+1));
//				if (nHot[iHot].fl == 1 && nHot[iHot].dLik > maxdLik + 0.1)	// here 0.1 is used to encourage longer hotspot while with the same starting point
//				{
//					maxI = iHot;
//					maxdLik = nHot[iHot].dLik;
//				}
//			}
//		}
//
//		if (DEBUG != 2)
//		{
//			cout << "Potential hotspot that maximize delta likelihood is:\t"
//				 << "start = " << nHot[maxI].gint.st << "\tend = "<< nHot[maxI].gint.ed << "\tdelta likelihood = " << nHot[maxI].dLik << endl;
//		}
//
//		//////////////////////////////////////////////////////////////////////////
//		// update if necessary
//		if (nHot[maxI].dLik >= ww.thrd)
//		{
//			cout << "Start = " << gr.pos[nHot[maxI].gint.st] << ",\tEnd = " << gr.pos[nHot[maxI].gint.ed] << ",\t"
//				 << "Delta likelihood = " << nHot[maxI].dLik << endl
//				 << "The new hotspot has been accepted." << endl;
//			hr.hInter[hr.numHot].st = nHot[maxI].gint.st;
//			hr.hInter[hr.numHot].ed = nHot[maxI].gint.ed;
//			for (i = nHot[maxI].gint.st; i < nHot[maxI].gint.ed; i++)
//			{
//				hr.deltaLik[i] = nHot[maxI].dLik;
//			}
//			hr.dLik[hr.numHot] = nHot[maxI].dLik;
//			hr.numHot++;
//			//////// recombination rates
//			for (i = 0; i < hr.numGrid-1; i++)
//			{
//				if (hr.hot[i] == 0)
//				{
//					hr.iRate[i] = nHot[maxI].bkRate;
//				}
//			}
//			for (i = nHot[maxI].gint.st; i < nHot[maxI].gint.ed; i++)
//			{
//				hr.iRate[i] = nHot[maxI].hotRate;
//			}
//			/////// background tags
//			/////////!!!!! whether to set its neighborhood as none background is under discussion
//			for (i = nHot[maxI].gint.st; i < nHot[maxI].gint.ed; i++)
//			{
//				hr.back[i] = 1;
//			}
//			/////// hotspot tags
//			for (i = nHot[maxI].gint.st; i < nHot[maxI].gint.ed; i++)
//			{
//				hr.hot[i] = iter + 1;
//			}
//			///// free tags
//			for (i = nHot[maxI].gint.st; i < nHot[maxI].gint.ed; i++)
//			{
//				hr.free[i] = 1;
//			}
//			if (nHot[maxI].gint.st != 0)
//			{
//				hr.free[nHot[maxI].gint.st - 1] = 1;
//			}
//			if (nHot[maxI].gint.ed != hr.numGrid-1)
//			{
//				hr.free[nHot[maxI].gint.ed] = 1;
//			}
//
//			//////////////////////////////////////////////////////////////////////////
//			// output results
//			if (DEBUG != 2)
//			{
//				cout << "New background tags are:" << endl;
//				for (i = 0; i < hr.numGrid-1; i++)
//				{
//					cout << "\t" << hr.back[i];
//				}
//				cout << endl;
//				cout << "New free to search tags are:" << endl;
//				for (i = 0; i < hr.numGrid-1; i++)
//				{
//					cout << "\t" << hr.free[i];
//				}
//				cout << endl;
//				cout << "New hot tags are:" << endl;
//				for (i = 0; i < hr.numGrid-1; i++)
//				{
//					cout << "\t" << hr.hot[i];
//				}
//				cout << endl;
//				cout << "New delta likelihoods are:" << endl;
//				for (i = 0; i < hr.numGrid-1; i++)
//				{
//					cout << "\t" << hr.deltaLik[i];
//				}
//				cout << endl;
//				cout << "New recombination rates are:" << endl;
//				for (i = 0; i < hr.numGrid-1; i++)
//				{
//					cout << "\t" << hr.iRate[i];
//				}
//				cout << endl;
//			}
//		}
//		else
//		{
//			cout << "Start = " << gr.pos[nHot[maxI].gint.st] << ",\tEnd = " << gr.pos[nHot[maxI].gint.ed] << ",\t"
//				 << "Delta likelihood = " << nHot[maxI].dLik << endl
//				 << "The new hotspot has been rejected." << endl;
//			hr.dLik[hr.numHot] = nHot[maxI].dLik;
//			cout << endl << "End of iteration." << endl;
//			break;
//		}
//	}
//
//	//////////////////////////////////////////////////////////////////////////
//	// release space
//	delete[] gr.pos;
//	delete[] gr.iRate;
//	delete[] sr.locs;
//	delete[] sr.aRate;
//	delete[] temphr.back;
//	delete[] temphr.free;
//	delete[] temphr.hot;
//	delete[] temphr.iRate;
//	delete[] nHot;
//}
